@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,14 +15,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: render_handler_agg.h,v 1.7 2006/10/15 10:10:23 nihilus Exp $ */
 
 #ifndef BACKEND_RENDER_HANDLER_AGG_H
 #define BACKEND_RENDER_HANDLER_AGG_H
 
+#include "dsodefs.h"
+
 namespace gnash {
 
 // Base class to shield GUIs from AGG's pixelformat classes 
@@ -40,7 +38,7 @@ public:
   virtual ~render_handler_agg_base() {}
   
   // these methods need to be accessed from outside:
-  virtual void init_buffer(unsigned char *mem, int size, int x, int y)=0;
+  virtual void init_buffer(unsigned char *mem, int size, int x, int y, int rowstride)=0;
 
   virtual unsigned int getBytesPerPixel() const=0;
 
@@ -51,7 +49,7 @@ public:
     
     _testBuffer	= static_cast<unsigned char *>( realloc(_testBuffer, size) );
     
-    init_buffer(_testBuffer, size, width, height);
+    init_buffer(_testBuffer, size, width, height, width * getBytesPerPixel());
     
     return true;
   }
@@ -71,7 +69,8 @@ DSOEXPORT render_handler_agg_base*
 /// Detect pixel format based on bit mask. If the pixel format is unknown,
 /// NULL is returned. Note that a successfully detected pixel format does
 /// not necessarily mean that the pixel format is available (compiled in).
-DSOEXPORT char* agg_detect_pixel_format(unsigned int rofs, unsigned int rsize,
+/// The bit offsets are assumed to be in host byte order!
+DSOEXPORT const char* agg_detect_pixel_format(unsigned int rofs, unsigned int rsize,
   unsigned int gofs, unsigned int gsize,
   unsigned int bofs, unsigned int bsize,
   unsigned int bpp);
