@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include <boost/algorithm/string/case_conv.hpp>
 
 #include <dirent.h> // used by scandir()
-#include <unistd.h> // used by unlink()
+#include "GnashSystemIOHeaders.h" // used by unlink()
 #include <fcntl.h>  // used by asyncmode()
 
 #include "VM.h"
@@ -35,7 +35,7 @@
 #include "as_object.h"
 #include "builtin_function.h" // need builtin_function
 #include "fileio.h"
-#include "array.h"  // used by scandir()
+#include "Array_as.h"  // used by scandir()
 
 using namespace std;
 
