@@ -1,6 +1,6 @@
-// Player.cpp:  Top level flash player, for gnash.
+// Player.cpp:  Top level SWF player, for gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,10 +31,12 @@
 #include "gnash.h" // still needed ?
 #include "movie_definition.h"
 #include "sound_handler.h" // for set_sound_handler and create_sound_handler_*
-#include "sprite_instance.h" // for setting FlashVars
+#include "MovieClip.h" // for setting FlashVars
 #include "movie_root.h" 
 #include "Player.h"
+#include "StreamProvider.h"
 
+#include "NamingPolicy.h"
 #include "StringPredicates.h"
 #include "URL.h"
 #include "rc.h"
@@ -43,11 +45,17 @@
 #include "VM.h"
 #include "SystemClock.h"
 
+#ifdef USE_FFMPEG
+# include "MediaHandlerFfmpeg.h"
+#elif defined(USE_GST)
+# include "MediaHandlerGst.h"
+#endif
+
+#include "GnashSystemIOHeaders.h" // for write() 
 #include "log.h"
 #include <iostream>
 #include <sstream>
 #include <iomanip>
-#include <unistd.h>
 
 using namespace gnash;
 
@@ -55,7 +63,7 @@ namespace {
 gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 }
 
-std::auto_ptr<Gui> Player::_gui(NULL);
+/// @todo Shouldn't Player be in 'gnash' namespace ?
 
 /*static private*/
 void
@@ -91,7 +99,8 @@ Player::Player()
     _fpsDebugTime(0.0),
 #endif
     _hostfd(-1),
-    _startFullscreen(false)
+    _startFullscreen(false),
+    _hideMenu(false)
 {
     init();
 }
@@ -110,12 +119,6 @@ Player::init()
     /// Initialize gnash core library
     gnashInit();
 
-    set_use_cache_files(false);
-
-    gnash::registerFSCommandCallback(fs_callback);
-    
-    gnash::movie_root::registerEventCallback(&interfaceEventCallback);
-
 }
 
 void
@@ -158,19 +161,26 @@ Player::init_logfile()
 
 }
 
-bool
-Player::silentStream(void* /*udata*/, boost::uint8_t* stream, int len)
+unsigned int
+Player::silentStream(void* /*udata*/, boost::int16_t* stream, unsigned int len, bool& atEOF)
 {
-    memset((void*)stream, 0, len);
-    return true;
+    std::fill(stream, stream+len, 0);
+    atEOF=false;
+    return len;
 }
 
 void
 Player::init_sound()
 {
+
     if (_doSound) {
 #ifdef SOUND_SDL
-        _soundHandler.reset( gnash::media::create_sound_handler_sdl(_audioDump) );
+        try {
+            _soundHandler.reset(sound::create_sound_handler_sdl(_audioDump));
+        } catch (SoundException& ex) {
+            log_error(_("Could not create sound handler: %s."
+                " Will continue w/out sound."), ex.what());
+        }
         if (! _audioDump.empty()) {
             // add a silent stream to the audio pool so that our output file
             // is homogenous;  we actually want silent wave data when no sounds
@@ -178,16 +188,29 @@ Player::init_sound()
             _soundHandler->attach_aux_streamer(silentStream, (void*) this);
         }
 #elif defined(SOUND_GST)
-        _soundHandler.reset( gnash::media::create_sound_handler_gst() );
+        _soundHandler.reset(media::create_sound_handler_gst());
 #else
         log_error(_("Sound requested but no sound support compiled in"));
         return;
 #endif
-        
-        gnash::set_sound_handler(_soundHandler.get());
     }
 }
 
+void
+Player::init_media()
+{
+#ifdef USE_FFMPEG
+        _mediaHandler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
+#elif defined(USE_GST)
+        _mediaHandler.reset( new gnash::media::gst::MediaHandlerGst() );
+#else
+        log_error(_("No media support compiled in"));
+        return;
+#endif
+        
+        gnash::media::MediaHandler::set(_mediaHandler);
+}
+
 
 void
 Player::init_gui()
@@ -195,13 +218,6 @@ Player::init_gui()
     if ( _doRender )
     {
         _gui = getGui(); 
-
-        RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-        if ( rcfile.startStopped() )
-        {
-            _gui->stop();
-        }
-
     }
     else
     {
@@ -222,6 +238,10 @@ Player::init_gui()
 boost::intrusive_ptr<movie_definition>
 Player::load_movie()
 {
+    /// The RunInfo must be initialized by this point to provide resources
+    /// for parsing.
+    assert(_runInfo.get());
+
     boost::intrusive_ptr<gnash::movie_definition> md;
 
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
@@ -239,8 +259,9 @@ Player::load_movie()
     try {
         if ( _infile == "-" )
         {
-            std::auto_ptr<tu_file> in ( noseek_fd_adapter::make_stream(fileno(stdin)) );
-            md = gnash::create_movie(in, _url, false);
+            std::auto_ptr<IOChannel> in (
+                    noseek_fd_adapter::make_stream(fileno(stdin)));
+            md = gnash::create_movie(in, _url, *_runInfo, false);
         }
         else
         {
@@ -260,7 +281,8 @@ Player::load_movie()
             }
 
             // _url should be always set at this point...
-            md = gnash::create_library_movie(url, _url.c_str(), false);
+            md = gnash::create_library_movie(url, *_runInfo, _url.c_str(),
+                    false);
         }
     } catch (const GnashException& er) {
         std::cerr << er.what() << std::endl;
@@ -278,13 +300,15 @@ Player::load_movie()
 
 /* \brief Run, used to open a new flash file. Using previous initialization */
 int
-Player::run(int argc, char* argv[], const std::string& infile, const std::string& url)
+Player::run(int argc, char* argv[], const std::string& infile,
+        const std::string& url)
 {
     
     // Call this at run() time, so the caller has
     // a cache of setting some parameter before calling us...
     // (example: setDoSound(), setWindowId() etc.. ) 
     init_logfile();
+    init_media();
     init_sound();
     init_gui();
    
@@ -293,7 +317,7 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
 
     _infile = infile;
 
-    // Set base url
+    // Work out base url
     if ( _baseurl.empty() )
     {
         if (! url.empty() ) _baseurl = url;
@@ -346,9 +370,17 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
         }
     }
 
-    // Set base url for this movie (needed before parsing)
-    if ( hasOverriddenBaseUrl ) gnash::set_base_url(URL(overriddenBaseUrl, URL(_baseurl)));
-    else gnash::set_base_url(URL(_baseurl));
+    URL baseURL = hasOverriddenBaseUrl ? URL(overriddenBaseUrl, URL(_baseurl))
+                                       : URL(_baseurl);
+
+    /// The RunInfo should be populated before parsing.
+    _runInfo.reset(new RunInfo(baseURL.str()));
+    _runInfo->setSoundHandler(_soundHandler);
+
+    std::auto_ptr<NamingPolicy> np(new IncrementalRename(_baseurl));
+    boost::shared_ptr<StreamProvider> sp(new StreamProvider(np));
+
+    _runInfo->setStreamProvider(sp);
 
     // Load the actual movie.
     _movieDef = load_movie();
@@ -361,7 +393,6 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     // Get info about the width & height of the movie.
     int movie_width = static_cast<int>(_movieDef->get_width_pixels());
     int movie_height = static_cast<int>(_movieDef->get_height_pixels());
-    float movie_fps = _movieDef->get_frame_rate();
 
     if (! _width) {
       _width = static_cast<size_t>(movie_width * _scale);
@@ -379,17 +410,41 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
         if ( ! _height ) _height = 1;
     }
 
+    // Register movie definition before creating the window
+    _gui->setMovieDefinition(_movieDef.get());
+
     // Now that we know about movie size, create gui window.
     _gui->createWindow(_url.c_str(), _width, _height);
 
     SystemClock clock; // use system clock here...
-    movie_root& root = VM::init(*_movieDef, clock).getRoot();
+    movie_root root(*_movieDef, clock, *_runInfo);
+
+    _callbacksHandler.reset(new CallbacksHandler(*_gui, *this)); 
+    
+    // Register Player to receive events from the core (Mouse, Stage,
+    // System etc)
+    root.registerEventCallback(_callbacksHandler.get());
+    
+    // Register Player to receive FsCommand events from the core.
+    root.registerFSCommandCallback(_callbacksHandler.get());
 
     // Set host requests fd (if any)
     if ( _hostfd != -1 ) root.setHostFD(_hostfd);
 
     _gui->setStage(&root);
 
+    // When startStopped is true, stop here after the stage has been 
+    // registered, but before the movie has started. Initial loading
+    // and VM initialization have been done by this stage, but not
+    // the complete parsing of the SWF. This is important because
+    // the Gui accesses movie_root to get the sound_handler, but also
+    // because the gui window should be properly set up by this point.
+    RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+    if ( rcfile.startStopped() )
+    {
+        _gui->stop();
+    }
+
     // Start loader thread
     // NOTE: the loader thread might (in IMPORT tag parsing)
     //       create new movies and register them to the MovieLibrary.
@@ -400,10 +455,10 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     //       intrusive_ptr...
     _movieDef->completeLoad();
 
-    _gui->setMovieDefinition(_movieDef.get());
-
     if (! _delay) {
-      _delay = static_cast<unsigned int>(1000 / movie_fps) ; // milliseconds per frame
+      //float movie_fps = _movieDef->get_frame_rate();
+      //_delay = static_cast<unsigned int>(1000 / movie_fps) ; // milliseconds per frame
+      _delay = 10; // 10ms per heart beat
     }
     _gui->setInterval(_delay);
 
@@ -414,9 +469,14 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     if (!_windowID && _startFullscreen) {
         _gui->setFullscreen();
     }
+
+    if (!_windowID && _hideMenu) {
+        _gui->hideMenu();
+    }
+    
     _gui->run();
 
-    std::cerr << "Main loop ended, cleaning up" << std::endl;
+    log_debug("Main loop ended, cleaning up");
 
     // Clean up as much as possible, so valgrind will help find actual leaks.
     gnash::clear();
@@ -424,21 +484,99 @@ Player::run(int argc, char* argv[], const std::string& infile, const std::string
     return EXIT_SUCCESS;
 }
 
-// static private
-// For handling notification callbacks from ActionScript. The callback is
-// always sent to a hosting application (i.e. if a file descriptor is
-// supplied). It is never acted on by Gnash when running as a plugin.
+
 void
-Player::fs_callback(gnash::sprite_instance* movie, const std::string& command,
-                                const std::string& args)
+Player::CallbacksHandler::error(const std::string& msg)
 {
-    log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
+    _gui.error(msg);
+}
+
+bool
+Player::CallbacksHandler::yesNo(const std::string& query)
+{
+    return _gui.yesno(query);
+}
+
+std::string
+Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
+{
+    if (event == "Mouse.hide") {
+        return _gui.showMouse(false) ? "true" : "false";
+    }
+
+    if (event == "Mouse.show")
+    {
+        return _gui.showMouse(true) ? "true" : "false";
+    }
+    
+    if (event == "Stage.displayState")
+    {
+        if (arg == "fullScreen") _gui.setFullscreen();
+        else if (arg == "normal") _gui.unsetFullscreen();
+        return "";
+    }
+
+    if (event == "Stage.scaleMode" || event == "Stage.align" )
+    {
+        _gui.updateStageMatrix();
+        return "";
+    }
+    
+    if (event == "System.capabilities.screenResolutionX")
+    {
+        std::ostringstream ss;
+        ss << _gui.getScreenResX();
+        return ss.str();
+    }
+
+    if (event == "System.capabilities.screenResolutionY")
+    {
+        std::ostringstream ss;
+        ss << _gui.getScreenResY();
+        return ss.str();
+    }
+
+    if (event == "System.capabilities.pixelAspectRatio")
+    {
+        std::ostringstream ss;
+        // Whether the pp actively limits the precision or simply
+        // gets a slightly different result isn't clear.
+        ss << std::setprecision(7) << _gui.getPixelAspectRatio();
+        return ss.str();
+    }
+
+    if (event == "System.capabilities.screenDPI")
+    {
+        std::ostringstream ss;
+        ss << _gui.getScreenDPI();
+        return ss.str();
+    }
+
+    if (event == "System.capabilities.screenColor")
+    {
+        return _gui.getScreenColor();
+    }
+
+    if (event == "System.capabilities.playerType")
+    {
+        return _gui.isPlugin() ? "PlugIn" : "StandAlone";
+    }
+
+    log_error(_("Unhandled callback %s with arguments %s"), event, arg);
+    return "";
+}
+
+void
+Player::CallbacksHandler::notify(const std::string& command,
+        const std::string& args)
+{
+    //log_debug(_("fs_callback(%p): %s %s"), (void*)movie, command, args);
 
     gnash::RcInitFile& rcfile = gnash::RcInitFile::getDefaultInstance();
 
     // it's _hostfd, but we're a static method...
-    int hostfd = VM::get().getRoot().getHostFD();
-    if ( hostfd != -1 )
+    const int hostfd = _player.getHostFD();
+    if (hostfd != -1)
     {
         //log_debug("user-provided host requests fd is %d", hostfd);
         std::stringstream request;
@@ -471,7 +609,7 @@ Player::fs_callback(gnash::sprite_instance* movie, const std::string& command,
 
     /// Fscommands can be ignored using an rcfile setting. As a 
     /// plugin they are always ignored.
-    if (_gui->isPlugin())
+    if (_gui.isPlugin())
     {
         // We log the request to the fd above
         log_debug(_("Running as plugin: skipping internal "
@@ -493,23 +631,23 @@ Player::fs_callback(gnash::sprite_instance* movie, const std::string& command,
     // FSCommand quit
     if (noCaseCompare(command, "quit"))
     {
-        _gui->quit();
+        _gui.quit();
         return;
     }
 
     // FSCommand fullscreen
     if (noCaseCompare(command, "fullscreen"))
     {
-        if (noCaseCompare(args, "true")) _gui->setFullscreen();
-        else if (noCaseCompare(args, "false")) _gui->unsetFullscreen();
+        if (noCaseCompare(args, "true")) _gui.setFullscreen();
+        else if (noCaseCompare(args, "false")) _gui.unsetFullscreen();
         return;
     }
        
     // FSCommand showmenu
     if (noCaseCompare(command, "showmenu"))
     {
-        if (noCaseCompare(args, "true")) _gui->showMenu(true);
-        else if (noCaseCompare(args, "false")) _gui->showMenu(false);
+        if (noCaseCompare(args, "true")) _gui.showMenu(true);
+        else if (noCaseCompare(args, "false")) _gui.showMenu(false);
         return;
     }
 
@@ -526,7 +664,13 @@ Player::fs_callback(gnash::sprite_instance* movie, const std::string& command,
     // FSCommand allowscale
     if (noCaseCompare(command, "allowscale"))
     {
-        log_unimpl(_("FsCommand allowscale called with argument %s"), args);
+        //log_debug("allowscale: %s", args);
+        if (noCaseCompare(args, "true")) _gui.allowScale(true);
+        else
+        {
+                if (strtol(args.c_str(), NULL, 0)) _gui.allowScale(true);
+                else _gui.allowScale(false);
+        }
         return;
     }
 
@@ -544,75 +688,6 @@ Player::fs_callback(gnash::sprite_instance* movie, const std::string& command,
 
 }
 
-std::string
-Player::interfaceEventCallback(const std::string& event, const std::string& arg)
-{
-    if (event == "Mouse.hide")
-    {
-        return _gui->showMouse(false) ? "true" : "false";
-    }
-
-    if (event == "Mouse.show")
-    {
-        return _gui->showMouse(true) ? "true" : "false";
-    }
-    
-    if (event == "Stage.displayState")
-    {
-        if (arg == "fullScreen") _gui->setFullscreen();
-        else if (arg == "normal") _gui->unsetFullscreen();
-        return "";
-    }
-
-    if (event == "Stage.scaleMode" || event == "Stage.align" )
-    {
-        _gui->updateStageMatrix();
-        return "";
-    }
-    
-    if (event == "System.capabilities.screenResolutionX")
-    {
-        std::ostringstream ss;
-        ss << _gui->getScreenResX();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.screenResolutionY")
-    {
-        std::ostringstream ss;
-        ss << _gui->getScreenResY();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.pixelAspectRatio")
-    {
-        std::ostringstream ss;
-        // Whether the pp actively limits the precision or simply
-        // gets a slightly different result isn't clear.
-        ss << std::setprecision(7) << _gui->getPixelAspectRatio();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.screenDPI")
-    {
-        std::ostringstream ss;
-        ss << _gui->getScreenDPI();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.screenColor")
-    {
-        return _gui->getScreenColor();
-    }
-
-    if (event == "System.capabilities.playerType")
-    {
-        return _gui->isPlugin() ? "PlugIn" : "StandAlone";
-    }
-
-    log_error(_("Unhandled callback %s with arguments %s"), event, arg);
-    return "";
-}
 
 // private
 std::auto_ptr<Gui>
@@ -622,10 +697,14 @@ Player::getGui()
     return createGTKGui(_windowID, _scale, _doLoop, _bitDepth);
 #endif
 
-#ifdef GUI_KDE
+#ifdef GUI_KDE3
     return createKDEGui(_windowID, _scale, _doLoop, _bitDepth);
 #endif
 
+#ifdef GUI_KDE4
+    return createKDE4Gui(_windowID, _scale, _doLoop, _bitDepth);
+#endif
+
 #ifdef GUI_SDL
     return createSDLGui(_windowID, _scale, _doLoop, _bitDepth);
 #endif
@@ -653,3 +732,11 @@ Player::getGui()
     return std::auto_ptr<Gui>(new NullGui(_doLoop));
 }
 
+Player::~Player()
+{
+    if (_movieDef.get())
+    {
+        log_debug("~Player - _movieDef refcount: %d (1 will be dropped "
+                "now)", _movieDef->get_ref_count());
+    }
+}
