@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,11 +18,7 @@
 #include "gtk_glue.h"
 
 #include <gtk/gtk.h>
-#ifndef _WIN32
-#include <gdk/gdkx.h>
-#else
 #include <gdk/gdk.h>
-#endif
 
 # include <cairo.h>
 
@@ -38,13 +34,19 @@ class GtkCairoGlue : public GtkGlue
     bool init(int argc, char ***argv);
     void prepDrawingArea(GtkWidget *drawing_area);
     render_handler* createRenderHandler();
+    void beforeRendering();
     void render();
     void render(int minx, int miny, int maxx, int maxy);
     void configure(GtkWidget *const widget, GdkEventConfigure *const event);
   private:
+    cairo_surface_t* createGdkImageSurface(const int& width, const int& height);
+    cairo_surface_t* createSimilarSurface(const int& width, const int& height);
+    cairo_surface_t* createMemorySurface(const int& width, const int& height);
+
     cairo_t     *_cairo_handle;
     cairo_t     *_cairo_offscreen;
     render_handler* _renderer;
+    GdkImage*       _image;
 };
 
 } // namespace gnash
