@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef _GUI_H_
-#define _GUI_H_
+#ifndef GNASH_GUI_H
+#define GNASH_GUI_H
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -24,16 +24,21 @@
 
 #include "rect.h"  // for composition
 #include "snappingrange.h"  // for InvalidatedRanges
-#include "gnash.h" // for gnash::key::code type
+#include "GnashKey.h" // for gnash::key::code type
 #include "smart_ptr.h"
 
 #ifdef USE_SWFTREE
 #include "tree.hh" // for tree
 #endif
 
+#include <cstdlib> // exit
 #include <string>
 #include <map>
 
+// Define this to enable fps debugging without touching
+// gnashconfig.h
+//#define GNASH_FPS_DEBUG
+
 // Define the following macro if you want to skip rendering
 // when late on FPS time.
 // This is an experimental feature, so it's off by default
@@ -61,6 +66,7 @@ namespace gnash
 {
     class render_handler;
     class movie_root;
+    class movie_definition;
 }
 
 namespace gnash
@@ -115,16 +121,9 @@ public:
     /// The default implementation calls exit(0), which isn't nice.
     /// Please implement the proper main loop quitter in the subclasses.
     ///
-    virtual void quit()  { exit(0); }
-
-    /// Create a menu and attach it to our window.
-    virtual bool createMenu() = 0;
-
-    /// Register event handlers.
-    virtual bool setupEvents() = 0;
+    virtual void quit()  { std::exit(0); }
 
     /// Render the current buffer.
-
     /// For OpenGL, this means that the front and back buffers are swapped.
     virtual void renderBuffer() = 0;
 
@@ -184,10 +183,10 @@ public:
     virtual std::string getScreenColor() { return ""; }
 
     /// @return Whether or not the movie should be looped indefinitely.
-    bool loops();
+    bool loops() const { return _loop; }
 
     /// @return Whether the movie is running fullscreen or not.    
-    bool isFullscreen() { return _fullscreen; }
+    bool isFullscreen() const { return _fullscreen; }
 
     /// Mouse notification callback to be called when the mouse is moved.
     //
@@ -219,7 +218,7 @@ public:
     /// @param height The desired height in pixels.
     void resize_view(int width, int height);
 
-    /// Update stage matrix accordingly to window size and flash Stage
+    /// Update stage SWFMatrix accordingly to window size and flash Stage
     /// configuration (scaleMode, alignment)
     //
     /// This method should be called from the core lib when Stage configuration
@@ -254,6 +253,10 @@ public:
     /// Return from fullscreen to normal mode.
     ///
     virtual void unsetFullscreen();
+
+    /// Hide the menu bar when using standalone player
+    ///
+    virtual void hideMenu();
     
     /// Sets whether the gui should show the system mouse pointer
     //
@@ -265,6 +268,11 @@ public:
     //
     /// @param show true if the menu bar should be shown.
     virtual void showMenu(bool show);
+
+    /// Sets whether scaling should be allowed (for fscommand)
+    //
+    /// @param allow true if stage scaling should be allowed
+    virtual void allowScale(bool allow);
     
     // Toggle between fullscreen and normal mode
     void toggleFullscreen();
@@ -299,29 +307,21 @@ public:
     
     /// Whether gnash is is running as a plugin
     bool isPlugin() const { return (( _xid )); }
-    
-    void setMaxAdvances(unsigned long ul) { if (ul > 0) _maxAdvances = ul; }
+
+    /// Set the maximum number of frame advances before Gnash exits.
+    void setMaxAdvances(unsigned long ul) { if (ul) _maxAdvances = ul; }
     
     void showUpdatedRegions(bool x) { _showUpdatedRegions = x; }
     bool showUpdatedRegions() { return _showUpdatedRegions; }
 
-    /** @name Menu callbacks
-     *  These callbacks will be called when a menu item is clicked.
-     *  @{
-     */
-    void menu_restart();
-    void menu_quit();
-    void menu_about();
-    void menu_play();
-    void menu_refresh_view();
-    void menu_pause();
-    void menu_stop();
-    void menu_step_forward();
-    void menu_step_backward();
-    void menu_jump_forward();
-    void menu_jump_backward();
-    void menu_toggle_sound();
-    /// @}
+    /// Instruct the core to restart the movie and
+    /// set state to play(). This does not change pause
+    /// state.
+    void restart();
+
+    /// Toggle sound state between muted and unmuted. If
+    /// there is no active sound handler this does nothing.
+    void toggleSound();
 
 #ifdef GNASH_FPS_DEBUG
     /// Set the interval between FPS debugging prints
@@ -359,6 +359,24 @@ public:
     /// Set the stage to advance/display
     void setStage(movie_root* stage);
 
+    /// Handle error message from the core
+    //
+    /// @param msg        The error message recieved
+    ///
+    virtual void error(const std::string& /*msg*/) {}
+
+    /// Prompt user with a question she can answer with yes/no
+    //
+    /// @param question
+    ///        The question to ask user
+    ///
+    /// @return
+    ///        true for YES, false for NO
+    ///
+    /// The default implementation always returns true.
+    ///
+    virtual bool yesno(const std::string& question);
+
 protected:
 
     /// Default constructor. Initialises members to safe defaults.
@@ -379,8 +397,10 @@ protected:
      * @param depth Colour depth to be used in the client area of our window.
      */
     Gui(unsigned long xid, float scale, bool loop, unsigned int depth);
+
     /// Determines if playback should restart after the movie ends.
     bool            _loop;
+
     /// The X Window ID to attach to. If zero, we create a new window.
     unsigned long   _xid;
 
@@ -398,23 +418,33 @@ protected:
 
     /// Desired colour depth in bits.
     int             _depth;
+
     /// Main loop interval: the time between successive advance_movie calls.
     unsigned int    _interval;
+
     /// The handler which is called to update the client area of our window.
     render_handler* _renderer;
+
     /// Signals that the next frame must be re-rendered completely because the
     /// window size did change.
     bool            _redraw_flag;
+
     // True if Gnash is running in fullscreen
     bool _fullscreen;
+
     // True if mouse pointer is showing
     bool _mouseShown;
+
     // Maximum number of advances before exit; 0 for no limit.
     unsigned long _maxAdvances;
+    
+    /// Counter to keep track of frame advances
+    unsigned long _advances;
 
     /// Called by Gui::stop().  This can be used by GUIs to implement pause
     /// widgets (so that resuming a stopped animation is more user-friendly)
     virtual void stopHook() {}
+
     /// Called by Gui::play().
     virtual void playHook() {}
 
@@ -427,10 +457,10 @@ private:
     float           _yscale;
 
     /// Window pixel X offset of stage origin
-    float           _xoffset;
+    boost::int32_t   _xoffset;
 
     /// Window pixel Y offset of stage origin
-    float           _yoffset;
+    boost::int32_t   _yoffset;
 
     bool display(movie_root* m);
     
@@ -469,7 +499,8 @@ private:
     VariableMap _flashVars;
 
     boost::intrusive_ptr<movie_definition> _movieDef;
-
+    
+    /// The root movie, or "Stage"
     movie_root* _stage;
 
     /// True if the application has been put into "stop" mode
@@ -477,7 +508,7 @@ private:
 
     /// True if the application didn't start yet
     bool            _started;
-    
+
     /// If true, updated regions (invalidated ranges) are visibly outlined.
     bool _showUpdatedRegions;
     
@@ -493,6 +524,7 @@ private:
 /// Named constructors
 std::auto_ptr<Gui> createGTKGui(unsigned long xid, float scale, bool loop, unsigned int depth);
 std::auto_ptr<Gui> createKDEGui(unsigned long xid, float scale, bool loop, unsigned int depth);
+std::auto_ptr<Gui> createKDE4Gui(unsigned long xid, float scale, bool loop, unsigned int depth);
 std::auto_ptr<Gui> createSDLGui(unsigned long xid, float scale, bool loop, unsigned int depth);
 std::auto_ptr<Gui> createFLTKGui(unsigned long xid, float scale, bool loop, unsigned int depth);
 std::auto_ptr<Gui> createFBGui(unsigned long xid, float scale, bool loop, unsigned int depth);
