@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef __KDESUP_H__
-#define __KDESUP_H__
+#ifndef GNASH_KDESUP_H
+#define GNASH_KDESUP_H
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
@@ -74,10 +74,6 @@ public slots:
     void menuitem_play_callback();
     void menuitem_pause_callback();
     void menuitem_stop_callback();
-    void menuitem_step_forward_callback();
-    void menuitem_step_backward_callback();
-    void menuitem_jump_forward_callback();
-    void menuitem_jump_backward_callback();
 private:
     QPopupMenu    _qmenu;
     KdeGui*       _godfather;
