@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,11 +15,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #include "gnash.h"
 
+#include <boost/cstdint.hpp> // for boost::?int??_t 
+
 namespace gnash
 {
 
@@ -29,7 +28,7 @@ class SdlGlue
     virtual ~SdlGlue() { }
     virtual bool init(int argc, char **argv[]) = 0;
 
-    virtual bool prepDrawingArea(int width, int height, uint32_t sdl_flags) = 0;
+    virtual bool prepDrawingArea(int width, int height, boost::uint32_t sdl_flags) = 0;
     virtual render_handler* createRenderHandler(int depth) = 0;
     virtual void render() = 0;
   protected:
