@@ -1,6 +1,6 @@
 // dlist.cpp:  Display lists, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include "log.h"
 #include "render.h"
 #include "StringPredicates.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 
 #include <typeinfo>
 #include <iostream>
@@ -177,15 +177,14 @@ DisplayList::get_character_by_name_i(const std::string& name)
 }
 
 void
-DisplayList::place_character(character* ch, int depth)
+DisplayList::place_character(character* ch, int depth, as_object* initObj)
 {
   assert(!ch->isUnloaded());
   ch->set_invalidated();
   ch->set_depth(depth);
 
   container_type::iterator it = std::find_if(
-      _charsByDepth.begin(), _charsByDepth.end(),
-      DepthGreaterOrEqual(depth));
+      _charsByDepth.begin(), _charsByDepth.end(), DepthGreaterOrEqual(depth));
 
   if ( it == _charsByDepth.end() || (*it)->get_depth() != depth )
   {
@@ -219,7 +218,7 @@ DisplayList::place_character(character* ch, int depth)
   }
 
   // Give life to this instance
-  ch->stagePlacementCallback();
+  ch->stagePlacementCallback(initObj);
 
   testInvariant();
 }
@@ -230,8 +229,8 @@ DisplayList::add(character* ch, bool replace)
   int depth = ch->get_depth();
 
   container_type::iterator it = std::find_if(
-      _charsByDepth.begin(), _charsByDepth.end(),
-      DepthGreaterOrEqual(depth));
+      _charsByDepth.begin(), _charsByDepth.end(), DepthGreaterOrEqual(depth));
+
   if ( it == _charsByDepth.end() || (*it)->get_depth() != depth )
   {
     _charsByDepth.insert(it, DisplayItem(ch));
@@ -274,14 +273,15 @@ DisplayList::replace_character(
   ch->set_invalidated();
   ch->set_depth(depth);
 
-  // NOTE: currently, ::restart also cleans up all property, which include __proto__ !!
-  //       For this reason I commented it out. Since no tests in the testsuite are failing
-  //       I'm not sure what does this break. Udo: do you remember ? --strk;
+  // NOTE: currently, ::restart also cleans up all
+  // property, which include __proto__ !!
+  // For this reason I commented it out. Since no tests in
+  // the testsuite are failing
+  // I'm not sure what does this break. Udo: do you remember ? --strk;
   // ch->restart();
 
   container_type::iterator it = std::find_if(
-      _charsByDepth.begin(), _charsByDepth.end(),
-      DepthGreaterOrEqual(depth));
+      _charsByDepth.begin(), _charsByDepth.end(), DepthGreaterOrEqual(depth));
 
   DisplayItem di(ch);
 
@@ -304,8 +304,8 @@ DisplayList::replace_character(
 
     if (use_old_matrix)
     {
-      // Use the matrix from the old character.
-      ch->copyMatrix(*oldch); // copy matrix and caches
+      // Use the SWFMatrix from the old character.
+      ch->copyMatrix(*oldch); // copy SWFMatrix and caches
     }
     
     // remember bounds of old char
@@ -329,7 +329,7 @@ DisplayList::replace_character(
     // WARNING: when a new Button character is added,
     //          the invalidated bounds computation will likely
     //          be bogus, as the actual character shown is not instantiated
-    //          until ::stagePlacementCallback for buttons (I'd say this is a bug in button_character_instance)
+    //          until ::stagePlacementCallback for buttons (I'd say this is a bug in Button)
     //          UdoG, following ? 
     //
     ch->extend_invalidated_bounds(old_ranges);        
@@ -349,7 +349,7 @@ void
 DisplayList::move_character(
   int depth,
   const cxform* color_xform,
-  const matrix* mat,
+  const SWFMatrix* mat,
   int* ratio,
   int* /* clip_depth */)
 {
@@ -390,7 +390,7 @@ DisplayList::move_character(
   }
   if (mat)
   {
-    ch->set_matrix(*mat, true); // update matrix caches
+    ch->setMatrix(*mat, true); // update SWFMatrix caches
   }
   if(ratio)
   {
@@ -616,7 +616,7 @@ DisplayList::display()
         character* ch = it->get();
 
         character* mask = ch->getMask();
-        if ( mask && ch->get_visible() && ! mask->isUnloaded() )
+        if ( mask && ch->isVisible() && ! mask->isUnloaded() )
         {
             render::begin_submit_mask();
             
@@ -658,7 +658,7 @@ DisplayList::display()
         }
         
         // check for non-mask hiden characters
-        if( !renderAsMask && (ch->get_visible() == false))
+        if( !renderAsMask && (!ch->isVisible()))
         {
             ch->omit_display();
             // Don't display non-mask hidden characters
@@ -866,25 +866,6 @@ DisplayList::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
 }
 
 
-void 
-DisplayList::dump_character_tree(const std::string prefix) const
-{
-  // print self:
-  //character::dump_character_tree(prefix);
-
-  // recursion:
-  for( const_iterator it = _charsByDepth.begin(),
-      endIt = _charsByDepth.end();
-    it != endIt; ++it)
-  {
-    const DisplayItem& dobj = *it;
-    
-    dobj->dump_character_tree(prefix+" "); 
-  }
-  
-
-}
-
 /// This method is not in the header in the hope DisplayItemDepthLess
 /// will be inlined by compiler
 
@@ -977,11 +958,11 @@ DisplayList::mergeDisplayList(DisplayList & newList)
                 {
                     newList._charsByDepth.erase(itNewBackup);
 
-                    // replace the transformation matrix if the old character accepts 
+                    // replace the transformation SWFMatrix if the old character accepts 
 					// static transformation.
                     if( chOld->get_accept_anim_moves() )
                     {
-                        chOld->copyMatrix(*chNew); // copy matrix and caches 
+                        chOld->copyMatrix(*chNew); // copy SWFMatrix and caches 
                         chOld->set_cxform(chNew->get_cxform());
                     }
                     chNew->unload();
