@@ -1,6 +1,6 @@
 // impl.cpp:  Implement ActionScript tags, movie loading, library, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,32 +22,39 @@
 #include "IOChannel.h"
 #include "utility.h"
 #include "impl.h"
-#include "font.h"
 #include "fontlib.h"
 #include "log.h"
-#include "image.h"
+#include "GnashImage.h"
 #include "sprite_definition.h"
 #include "SWFMovieDefinition.h"
 #include "swf.h"
 #include "swf/TagLoadersTable.h"
 #include "URL.h"
 #include "StreamProvider.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "VM.h"
 #include "ScriptLimitsTag.h"
 #include "BitmapMovieDefinition.h"
 #include "DefineFontAlignZonesTag.h"
 #include "DefineButtonCxformTag.h"
 #include "CSMTextSettingsTag.h"
+#include "DefineFontTag.h"
+#include "DefineButtonTag.h"
+#include "DefineTextTag.h"
 #include "PlaceObject2Tag.h"
 #include "RemoveObjectTag.h"
 #include "DoActionTag.h"
 #include "DoInitActionTag.h"
+#include "DefineEditTextTag.h"
 #include "SetBackgroundColorTag.h"
 #include "StartSoundTag.h"
 #include "StreamSoundBlockTag.h"
+#include "DefineButtonSoundTag.h"
+#include "DefineVideoStreamTag.h"
+#include "DefineFontNameTag.h"
+#include "VideoFrameTag.h"
 #include "swf/tag_loaders.h" // for all tag loaders..
-#include "sound_handler.h" // for get_sound_handler
+#include "RunInfo.h"
 #ifdef GNASH_USE_GC
 #include "GC.h"
 #endif
@@ -57,54 +64,24 @@
 #include <memory> // for auto_ptr
 #include <algorithm>
 
+
 namespace gnash
 {
 
+// Forward declarations
+namespace {
+    FileType getFileType(IOChannel& in);
+    SWFMovieDefinition* createSWFMovie(std::auto_ptr<IOChannel> in,
+            const std::string& url, const RunInfo& runInfo,
+            bool startLoaderThread);
+}
+
 static void clear_library();
 
-/// Namespace for global data (will likely turn into a class)
-namespace globals { // gnash::globals
-
-  /// global StreamProvider
-  StreamProvider& streamProvider = StreamProvider::getDefaultInstance();
-
-  /// Base url (for relative urls resolution)
-  //
-  /// we need an auto_ptr becase the URL class
-  /// is an immutable one and needs to be set 
-  /// at construction time..
-  ///
-  static std::auto_ptr<URL> baseurl;
-
-} // namespace gnash::global
-
-// global Sound handler stuff. Should this be moved to the VM class ?
-static media::sound_handler* _sound_handler = 0;
-void  set_sound_handler(media::sound_handler* s) { _sound_handler = s; }
-media::sound_handler* get_sound_handler() { return _sound_handler; }
-
-void
-set_base_url(const URL& url)
-{
-  // can call this only once during a single run
-  assert(!globals::baseurl.get());
-  globals::baseurl.reset(new URL(url));
-  log_debug(_("Base url set to: %s"), globals::baseurl->str());
-}
-
-const URL&
-get_base_url()
-{
-  // Don't call me if you haven't set me !
-  assert(globals::baseurl.get());
-  return *globals::baseurl;
-}
-
-
 // Associate the specified tag type with the given tag loader
 // function.
 void
-register_tag_loader(SWF::tag_type t, SWF::TagLoadersTable::loader_function lf)
+register_tag_loader(SWF::TagType t, SWF::TagLoadersTable::loader_function lf)
 {
   using SWF::TagLoadersTable;
 
@@ -135,22 +112,28 @@ static void ensure_loaders_registered()
     register_tag_loader(SWF::PLACEOBJECT, PlaceObject2Tag::loader);
     register_tag_loader(SWF::REMOVEOBJECT,  RemoveObjectTag::loader); // 05
     register_tag_loader(SWF::DEFINEBITS,  define_bits_jpeg_loader);
-    register_tag_loader(SWF::DEFINEBUTTON,  button_character_loader);
-    register_tag_loader(SWF::JPEGTABLES,  jpeg_tables_loader);
+    register_tag_loader(SWF::DEFINEBUTTON, DefineButtonTag::loader);
+    register_tag_loader(SWF::JPEGTABLES, jpeg_tables_loader);
     register_tag_loader(SWF::SETBACKGROUNDCOLOR, SetBackgroundColorTag::loader);
-    register_tag_loader(SWF::DEFINEFONT,  define_font_loader);
-    register_tag_loader(SWF::DEFINETEXT,  define_text_loader);
+    register_tag_loader(SWF::DEFINEFONT, DefineFontTag::loader);
+    register_tag_loader(SWF::DEFINETEXT, DefineTextTag::loader);
     register_tag_loader(SWF::DOACTION,  DoActionTag::doActionLoader);
-    register_tag_loader(SWF::DEFINEFONTINFO, define_font_info_loader);
-    register_tag_loader(SWF::DEFINEFONTINFO2, define_font_info_loader); // 62
+    register_tag_loader(SWF::DEFINEFONTINFO, DefineFontInfoTag::loader);
+    // 62
+    register_tag_loader(SWF::DEFINEFONTINFO2, DefineFontInfoTag::loader);
     register_tag_loader(SWF::DEFINESOUND, define_sound_loader);
-    register_tag_loader(SWF::STARTSOUND,  StartSoundTag::loader);
+    register_tag_loader(SWF::STARTSOUND, StartSoundTag::loader);
+    // 89
+    register_tag_loader(SWF::STARTSOUND2, StartSound2Tag::loader);
 
-    register_tag_loader(SWF::STOPSOUND,     fixme_loader); // 16 
+    register_tag_loader(SWF::STOPSOUND, fixme_loader); // 16 
 
-    register_tag_loader(SWF::DEFINEBUTTONSOUND, button_sound_loader);
-    register_tag_loader(SWF::SOUNDSTREAMHEAD, sound_stream_head_loader); // 18
-    register_tag_loader(SWF::SOUNDSTREAMBLOCK, StreamSoundBlockTag::loader); // 19
+    // 17
+    register_tag_loader(SWF::DEFINEBUTTONSOUND, DefineButtonSoundTag::loader);
+    // 18
+    register_tag_loader(SWF::SOUNDSTREAMHEAD, sound_stream_head_loader);
+    // 19
+    register_tag_loader(SWF::SOUNDSTREAMBLOCK, StreamSoundBlockTag::loader);
     register_tag_loader(SWF::DEFINELOSSLESS, define_bits_lossless_2_loader);
     register_tag_loader(SWF::DEFINEBITSJPEG2, define_bits_jpeg2_loader);
     register_tag_loader(SWF::DEFINESHAPE2,  define_shape_loader);
@@ -166,11 +149,12 @@ static void ensure_loaders_registered()
     // 30 - _UNKNOWN_ unimplemented
     register_tag_loader(SWF::FREEALL, fixme_loader); // 31
     register_tag_loader(SWF::DEFINESHAPE3,  define_shape_loader);
-    register_tag_loader(SWF::DEFINETEXT2, define_text_loader);
-    register_tag_loader(SWF::DEFINEBUTTON2, button_character_loader);
+    register_tag_loader(SWF::DEFINETEXT2, DefineText2Tag::loader);
+    // 37
+    register_tag_loader(SWF::DEFINEBUTTON2, DefineButton2Tag::loader);
     register_tag_loader(SWF::DEFINEBITSJPEG3, define_bits_jpeg3_loader);
     register_tag_loader(SWF::DEFINELOSSLESS2, define_bits_lossless_2_loader);
-    register_tag_loader(SWF::DEFINEEDITTEXT, define_edit_text_loader);
+    register_tag_loader(SWF::DEFINEEDITTEXT, DefineEditTextTag::loader);
     register_tag_loader(SWF::DEFINEVIDEO, fixme_loader); // 38
     register_tag_loader(SWF::DEFINESPRITE,  sprite_loader);
     register_tag_loader(SWF::NAMECHARACTER, fixme_loader); // 40
@@ -184,7 +168,8 @@ static void ensure_loaders_registered()
     register_tag_loader(SWF::SOUNDSTREAMHEAD2, sound_stream_head_loader); // 45
     register_tag_loader(SWF::DEFINEMORPHSHAPE, define_shape_morph_loader);
     register_tag_loader(SWF::FRAMETAG,  fixme_loader); // 47
-    register_tag_loader(SWF::DEFINEFONT2, define_font_loader); // 48
+    // 48
+    register_tag_loader(SWF::DEFINEFONT2, DefineFontTag::loader);
     register_tag_loader(SWF::GENCOMMAND,  fixme_loader); // 49
     register_tag_loader(SWF::DEFINECOMMANDOBJ, fixme_loader); // 50
     register_tag_loader(SWF::CHARACTERSET,  fixme_loader); // 51
@@ -204,8 +189,10 @@ static void ensure_loaders_registered()
 
     register_tag_loader(SWF::INITACTION, DoInitActionTag::doInitActionLoader);  // 59  
 
-    register_tag_loader(SWF::DEFINEVIDEOSTREAM, define_video_loader); // 60
-    register_tag_loader(SWF::VIDEOFRAME, video_loader); // 61
+    // 60
+    register_tag_loader(SWF::DEFINEVIDEOSTREAM, DefineVideoStreamTag::loader);
+    // 61
+    register_tag_loader(SWF::VIDEOFRAME, VideoFrameTag::loader);
 
     // 62, DEFINEFONTINFO2 is done above.
     // We're not an authoring tool.
@@ -232,7 +219,8 @@ static void ensure_loaders_registered()
     register_tag_loader(SWF::DEFINEALIGNZONES, DefineFontAlignZonesTag::loader); // 73
 
     register_tag_loader(SWF::CSMTEXTSETTINGS, CSMTextSettingsTag::loader); // 74
-    register_tag_loader(SWF::DEFINEFONT3, define_font_loader); // 75
+    // 75
+    register_tag_loader(SWF::DEFINEFONT3, DefineFontTag::loader);
     register_tag_loader(SWF::SYMBOLCLASS, fixme_loader); // 76 
     register_tag_loader(SWF::METADATA, metadata_loader); // 77
     register_tag_loader(SWF::DEFINESCALINGGRID, fixme_loader); // 78
@@ -240,16 +228,19 @@ static void ensure_loaders_registered()
     register_tag_loader(SWF::DEFINESHAPE4, define_shape_loader); // 83
     register_tag_loader(SWF::DEFINEMORPHSHAPE2, define_shape_morph_loader); // 84
     register_tag_loader(SWF::DEFINESCENEANDFRAMELABELDATA,define_scene_frame_label_loader); //86
-    register_tag_loader(SWF::DEFINEFONTNAME, define_font_name_loader); // 88
+    // 88
+    register_tag_loader(SWF::DEFINEFONTNAME, DefineFontNameTag::loader);
 
     register_tag_loader(SWF::REFLEX, reflex_loader); // 777
 }
 
-// Create a movie_definition from a png stream
-// NOTE: this method assumes this *is* a png stream
+// Create a movie_definition from an image format stream
+// NOTE: this method assumes this *is* the format described in the
+// FileType type
 // TODO: The pp won't display PNGs for SWF7 or below.
 static movie_definition*
-createBitmapMovie(std::auto_ptr<IOChannel> in, const std::string& url, FileType type)
+createBitmapMovie(std::auto_ptr<IOChannel> in, const std::string& url,
+        FileType type)
 {
     assert (in.get());
 
@@ -259,7 +250,9 @@ createBitmapMovie(std::auto_ptr<IOChannel> in, const std::string& url, FileType
 
     try
     {
-        std::auto_ptr<image::ImageBase> im(image::readImageData(imageData, type));
+        std::auto_ptr<GnashImage> im(
+                ImageInput::readImageData(imageData, type));
+
         if (!im.get())
         {
             log_error(_("Can't read image file from %s"), url);
@@ -278,119 +271,17 @@ createBitmapMovie(std::auto_ptr<IOChannel> in, const std::string& url, FileType
 
 }
 
-// Get type of file looking at first bytes
-// return "jpeg", "png", "swf" or "unknown"
-//
-FileType
-getFileType(IOChannel* in)
-{
-  in->seek(0);
-
-  unsigned char buf[3];
-  
-  if ( 3 < in->read(buf, 3) )
-  {
-    log_error(_("Can't read file header"));
-    in->seek(0);
-    return GNASH_FILETYPE_UNKNOWN;
-  }
-  
-  // This is the magic number for any JPEG format file
-  if ((buf[0] == 0xff) && (buf[1] == 0xd8) && (buf[2] == 0xff))
-  {
-    in->seek(0);
-    return GNASH_FILETYPE_JPEG;
-  }
-
-  // This is the magic number for any PNG format file
-  if ((buf[0] == 137) && (buf[1] == 'P') && (buf[2] == 'N')) // buf[3] == 'G' (we didn't read so far)
-  {
-    in->seek(0);
-    return GNASH_FILETYPE_PNG;
-  }
-
-  // This is the magic number for any GIF format file
-  if ((buf[0] == 'G') && (buf[1] == 'I') && (buf[2] == 'F'))
-  {
-    in->seek(0);
-    return GNASH_FILETYPE_GIF;
-  }
-
-  // This is for SWF (FWS or CWS)
-  if (  (buf[0] == 'F' || buf[0] == 'C') &&
-    (buf[1] == 'W') &&
-    (buf[2] == 'S') )
-  {
-    in->seek(0);
-    return GNASH_FILETYPE_SWF;
-  }
-
-  if ((buf[0] == 'F') && (buf[1] == 'L') && (buf[2] == 'V') ) {
-    return GNASH_FILETYPE_FLV;
-  }
-  
-  // Check if it is an swf embedded in a player (.exe-file)
-  if ((buf[0] == 'M') && (buf[1] == 'Z')) {
-
-    if ( 3 < in->read(buf, 3) )
-    {
-      log_error(_("Can't read 3 bytes after an MZ (.exe) header"));
-      in->seek(0);
-      return GNASH_FILETYPE_UNKNOWN;
-    }
-
-    while ((buf[0]!='F' && buf[0]!='C') || buf[1]!='W' || buf[2]!='S')
-    {
-      buf[0] = buf[1];
-      buf[1] = buf[2];
-      buf[2] = in->read_byte();
-      if (in->eof())
-      {
-        log_error(_("Could not find SWF inside an exe file"));
-        in->seek(0);
-        return GNASH_FILETYPE_UNKNOWN;
-      }
-    }
-    in->seek(in->tell()-3); // position to start of the swf itself
-    return GNASH_FILETYPE_SWF;
-  }
-
-  log_error("unknown file type, buf is %c%c%c", buf[0], buf[1], buf[2]);
-  return GNASH_FILETYPE_UNKNOWN;
-}
-
-// Create a SWFMovieDefinition from an SWF stream
-// NOTE: this method assumes this *is* an SWF stream
-//
-static SWFMovieDefinition*
-create_swf_movie(std::auto_ptr<IOChannel> in, const std::string& url, bool startLoaderThread)
-{
-
-  // Avoid leaks on error 
-  std::auto_ptr<SWFMovieDefinition> m ( new SWFMovieDefinition() );
-
-  if ( ! m->readHeader(in, url) )
-  {
-    return NULL;
-  }
-
-  if ( startLoaderThread && ! m->completeLoad() )
-  {
-    return NULL;
-  }
-
-  return m.release();
-}
 
 movie_definition*
-create_movie(std::auto_ptr<IOChannel> in, const std::string& url, bool startLoaderThread)
+create_movie(std::auto_ptr<IOChannel> in, const std::string& url,
+        const RunInfo& runInfo, bool startLoaderThread)
 {
   assert(in.get());
 
   ensure_loaders_registered();
 
   // see if it's a jpeg or an swf
-  FileType type = getFileType(in.get());
+  FileType type = getFileType(*in);
 
     switch (type)
     {
@@ -410,11 +301,12 @@ create_movie(std::auto_ptr<IOChannel> in, const std::string& url, bool startLoad
 
 
         case GNASH_FILETYPE_SWF:
-            return create_swf_movie(in, url, startLoaderThread);
+            return createSWFMovie(in, url, runInfo, startLoaderThread);
 
         case GNASH_FILETYPE_FLV:
             log_unimpl(_("FLV can't be loaded directly as a movie"));
             return NULL;
+
         default:
             log_error(_("unknown file type (%s)"), type);
             break;
@@ -424,40 +316,152 @@ create_movie(std::auto_ptr<IOChannel> in, const std::string& url, bool startLoad
 }
 
 movie_definition*
-create_movie(const URL& url, const char* reset_url, bool startLoaderThread, const std::string* postdata)
+create_movie(const URL& url, const RunInfo& runInfo, const char* reset_url,
+        bool startLoaderThread, const std::string* postdata)
 {
 
-  const std::string swfurl = url.str();
-
   std::auto_ptr<IOChannel> in;
-  if ( postdata ) in = globals::streamProvider.getStream(url, *postdata);
-  else in = globals::streamProvider.getStream(url);
+
+  const StreamProvider& streamProvider = runInfo.streamProvider();
+
+  const RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+
+  if (postdata) {
+      in = streamProvider.getStream(url, *postdata, rcfile.saveLoadedMedia());
+  }
+  else in = streamProvider.getStream(url, rcfile.saveLoadedMedia());
+
   if ( ! in.get() )
   {
-      log_error(_("failed to open '%s'; can't create movie"), swfurl);
+      log_error(_("failed to open '%s'; can't create movie"), url);
       return NULL;
   }
-  else if ( in->get_error() )
+  
+  if (in->bad())
   {
-      log_error(_("streamProvider opener can't open '%s'"), swfurl);
+      log_error(_("streamProvider opener can't open '%s'"), url);
       return NULL;
   }
 
-  const char* movie_url = reset_url ? reset_url : swfurl.c_str();
-  movie_definition* ret = create_movie(in, movie_url, startLoaderThread);
+  std::string urlstr = url.str();
+  const char* movie_url = reset_url ? reset_url : urlstr.c_str();
+  movie_definition* ret = create_movie(in, movie_url, runInfo,
+          startLoaderThread);
 
   return ret;
 
 
 }
 
+
+namespace {
+
+/// Get type of file looking at first bytes
+FileType
+getFileType(IOChannel& in)
+{
+    in.seek(0);
+
+    char buf[3];
+    
+    if (in.read(buf, 3) < 3)
+    {
+        log_error(_("Can't read file header"));
+        in.seek(0);
+        return GNASH_FILETYPE_UNKNOWN;
+    }
+    
+    // This is the magic number {0xff, 0xd8, 0xff} for JPEG format files
+    if (std::equal(buf, buf + 3, "\xff\xd8\xff"))
+    {
+        in.seek(0);
+        return GNASH_FILETYPE_JPEG;
+    }
+
+    // This is the magic number for any PNG format file
+    // buf[3] == 'G' (we didn't read so far)
+    if (std::equal(buf, buf + 3, "\x89PN")) 
+    {
+        in.seek(0);
+        return GNASH_FILETYPE_PNG;
+    }
+
+    // This is the magic number for any GIF format file
+    if (std::equal(buf, buf + 3, "GIF"))
+    {
+        in.seek(0);
+        return GNASH_FILETYPE_GIF;
+    }
+
+    // This is for SWF (FWS or CWS)
+    if (std::equal(buf, buf + 3, "FWS") || std::equal(buf, buf + 3, "CWS"))
+    {
+        in.seek(0);
+        return GNASH_FILETYPE_SWF;
+    }
+
+    // Take one guess at what this is. (It's an FLV-format file).
+    if (std::equal(buf, buf + 3, "FLV")) {
+        return GNASH_FILETYPE_FLV;
+    }
+    
+    // Check if it is an swf embedded in a player (.exe-file)
+    if (std::equal(buf, buf + 2, "MZ")) {
+
+        if ( 3 > in.read(buf, 3) )
+        {
+            log_error(_("Can't read 3 bytes after an MZ (.exe) header"));
+            in.seek(0);
+            return GNASH_FILETYPE_UNKNOWN;
+        }
+
+        while ((buf[0]!='F' && buf[0]!='C') || buf[1]!='W' || buf[2]!='S')
+        {
+            buf[0] = buf[1];
+            buf[1] = buf[2];
+            buf[2] = in.read_byte();
+            if (in.eof())
+            {
+                log_error(_("Could not find SWF inside an exe file"));
+                in.seek(0);
+                return GNASH_FILETYPE_UNKNOWN;
+            }
+        }
+        in.seek(in.tell() - static_cast<std::streamoff>(3));
+        return GNASH_FILETYPE_SWF;
+    }
+
+    log_error("unknown file type, buf is %c%c%c", buf[0], buf[1], buf[2]);
+    return GNASH_FILETYPE_UNKNOWN;
+}
+
+// Create a SWFMovieDefinition from an SWF stream
+// NOTE: this method assumes this *is* an SWF stream
+//
+SWFMovieDefinition*
+createSWFMovie(std::auto_ptr<IOChannel> in, const std::string& url,
+        const RunInfo& runInfo, bool startLoaderThread)
+{
+
+    std::auto_ptr<SWFMovieDefinition> m (new SWFMovieDefinition(runInfo));
+
+    const std::string& absURL = URL(url).str();
+
+    if (!m->readHeader(in, absURL)) return 0;
+    if (startLoaderThread && !m->completeLoad()) return 0;
+
+    return m.release();
+}
+
+}
+
 //
 // global gnash management
 //
 
 
+// Maximum release of resources.
 void  clear()
-    // Maximum release of resources.
 {
     // Ideally, we should make sure that function properly signals all threads
     // about exiting and giving them a chance to cleanly exit.
@@ -473,27 +477,20 @@ void  clear()
     //
     // See task task #6959 and depending items
     //
-    std::cerr << "Any segfault past this message is likely due to improper threads cleanup." << std::endl;
-    //exit(EXIT_SUCCESS);
+    log_debug("Any segfault past this message is likely due to improper "
+            "threads cleanup.");
+
+    VM::get().clear();
 
     clear_library();
     fontlib::clear();
 
-    if ( VM::isInitialized() )
-    {
-        VM::get().getRoot().clear();
-    }
-
 #ifdef GNASH_USE_GC 
     GC::get().collect();
 
     GC::cleanup();
 #endif
 
-    // By setting the soundhandler to NULL we avoid it being used
-    // after it's been de-referenced
-    set_sound_handler(NULL);
-
     // By setting the render handler to NULL we avoid it being used
     // after it's been de-referenced (fixes bug #21310)
     set_render_handler(NULL);
@@ -624,65 +621,57 @@ static void clear_library()
 // loaded it already.  Add it to our library on success, and
 // return a pointer to it.
 //
-movie_definition* create_library_movie(const URL& url, const char* real_url, bool startLoaderThread, const std::string* postdata)
+movie_definition* create_library_movie(const URL& url, const RunInfo& runInfo,
+        const char* real_url, bool startLoaderThread,
+        const std::string* postdata)
 {
-//    log_debug(_("%s: url is %s"), __PRETTY_FUNCTION__, url.str());
 
     // Use real_url as label for cache if available 
     std::string cache_label = real_url ? URL(real_url).str() : url.str();
 
     // Is the movie already in the library? (don't check if we have post data!)
-    if ( ! postdata )
+    if (!postdata)
     {
-  boost::intrusive_ptr<movie_definition>  m;
-  if ( s_movie_library.get(cache_label, &m) )
-      {
-        log_debug(_("Movie %s already in library"), cache_label);
-    return m.get();
-      }
+        boost::intrusive_ptr<movie_definition>  m;
+        if ( s_movie_library.get(cache_label, &m) )
+        {
+            log_debug(_("Movie %s already in library"), cache_label);
+            return m.get();
+        }
     }
 
-  // Try to open a file under the filename, but DO NOT start
-  // the loader thread now to avoid IMPORT tag loaders from 
-  // calling create_library_movie() again and NOT finding
-  // the just-created movie.
-  movie_definition* mov = create_movie(url, real_url, false, postdata);
-  //log_debug(_("create_movie(%s, %s, false) returned %p"), url.str(), real_url, mov);
+    // Try to open a file under the filename, but DO NOT start
+    // the loader thread now to avoid IMPORT tag loaders from 
+    // calling create_library_movie() again and NOT finding
+    // the just-created movie.
+    movie_definition* mov = create_movie(url, runInfo, real_url, false,
+            postdata);
 
-  if (mov == NULL)
-  {
-    log_error(_("Couldn't load library movie '%s'"),
-      url.str());
-    return NULL;
-  }
-
-  // Movie is good, add to the library 
-  if ( ! postdata ) // don't add if we POSTed
-  {
-  	s_movie_library.add(cache_label, mov);
-        log_debug(_("Movie %s (SWF%d) added to library"), cache_label, mov->get_version());
-  }
-  else
-  {
-        log_debug(_("Movie %s (SWF%d) NOT added to library (resulted from a POST)"), cache_label, mov->get_version());
-  }
-
-  // Now complete the load if the movie is an SWF movie
-  // 
-  // FIXME: add completeLoad() to movie_definition class
-  //        to allow loads of JPEG to use a loader thread
-  //        too...
-  //
-  if ( startLoaderThread )
-  {
-    SWFMovieDefinition* mdi = dynamic_cast<SWFMovieDefinition*>(mov);
-    if ( mdi ) {
-      mdi->completeLoad();
+    if (!mov)
+    {
+        log_error(_("Couldn't load library movie '%s'"), url.str());
+        return NULL;
     }
-  }
 
-  //log_debug(_("create_library_movie(%s, %s, startLoaderThread=%d) about to return %p"), url.str(), real_url, startLoaderThread, mov);
-  return mov;
+    // Movie is good, add to the library 
+    if (!postdata) // don't add if we POSTed
+    {
+        s_movie_library.add(cache_label, mov);
+        log_debug(_("Movie %s (SWF%d) added to library"),
+                cache_label, mov->get_version());
+    }
+    else
+    {
+        log_debug(_("Movie %s (SWF%d) NOT added to library (resulted from "
+                    "a POST)"), cache_label, mov->get_version());
+    }
+
+    /// Now complete the load if the movie is an SWF movie
+    // 
+    /// This is a no-op except for SWF movies.
+    if (startLoaderThread) mov->completeLoad();
+
+    return mov;
 }
 
 #ifdef GNASH_USE_GC
