@@ -1,6 +1,6 @@
 // movie_root.cpp:  The root movie, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,15 +17,17 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 // 
 
+#include "GnashSystemIOHeaders.h" // write()
+
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "movie_root.h"
 #include "log.h"
-#include "sprite_instance.h"
-#include "movie_instance.h" // for implicit upcast to sprite_instance
+#include "MovieClip.h"
+#include "movie_instance.h" // for implicit upcast to MovieClip
 #include "render.h"
 #include "VM.h"
 #include "ExecutableCode.h"
-#include "Stage.h"
+#include "Stage_as.h"
 #include "utility.h"
 #include "URL.h"
 #include "namedStrings.h"
@@ -33,8 +35,9 @@
 #include "sound_handler.h"
 #include "timers.h" // for Timer use
 #include "GnashKey.h" // key::code
-#include "../gui/gui.h"
+#include "gnash.h"
 
+#include <boost/algorithm/string/replace.hpp>
 #include <utility>
 #include <iostream>
 #include <string>
@@ -88,9 +91,12 @@ movie_root::testInvariant() const
 }
 
 
-movie_root::movie_root(VM& vm)
+movie_root::movie_root(const movie_definition& def,
+        VirtualClock& clock, const RunInfo& runInfo)
 	:
-    _vm(vm),
+    _runInfo(runInfo),
+    _originalURL(def.get_url()),
+    _vm(VM::init(def.get_version(), *this, clock)),
 	_interfaceHandler(0),
 	_fsCommandHandler(0),
 	m_viewport_x0(0),
@@ -98,16 +104,17 @@ movie_root::movie_root(VM& vm)
 	m_viewport_width(1),
 	m_viewport_height(1),
 	m_background_color(255, 255, 255, 255),
-        m_background_color_set(false),
+    m_background_color_set(false),
 	m_timer(0.0f),
 	m_mouse_x(0),
 	m_mouse_y(0),
 	m_mouse_buttons(0),
 	_lastTimerId(0),
-	m_active_input_text(NULL),
+	_currentFocus(0),
 	m_time_remainder(0.0f),
 	m_drag_state(),
 	_movies(),
+	_childs(),
 	_rootMovie(),
 	_invalidated(true),
 	_disableScripts(false),
@@ -118,10 +125,11 @@ movie_root::movie_root(VM& vm)
 	_displayState(normal),
 	_recursionLimit(256),
 	_timeoutLimit(15),
-	_gui(0),
 	_movieAdvancementDelay(83), // ~12 fps by default
 	_lastMovieAdvancement(0)
 {
+    // This takes care of informing the renderer (if present) too.
+    setQuality(QUALITY_HIGH);
 }
 
 void
@@ -194,18 +202,21 @@ movie_root::setRootMovie(movie_instance* movie)
 	{
 		setLevel(0, movie);
 
-		// actions in first frame of _level0 must execute now, before next advance,
+		// actions in first frame of _level0 must execute now,
+        // before next advance,
 		// or they'll be executed with _currentframe being set to 2
 		processActionQueue();
 	}
 	catch (ActionLimitException& al)
 	{
-		boost::format fmt = boost::format(_("ActionLimits hit during setRootMovie: %s. Disable scripts ?")) % al.what();
+		boost::format fmt = boost::format(_("ActionLimits hit during "
+                    "setRootMovie: %s. Disable scripts?")) % al.what();
 		handleActionLimitHit(fmt.str());
 	}
     catch (ActionParserException& e)
     {
-        log_error("ActionParserException thrown during setRootMovie: %s", e.what());
+        log_error("ActionParserException thrown during setRootMovie: %s",
+                e.what());
     }
 
 	cleanupAndCollect();
@@ -216,8 +227,9 @@ void
 movie_root::handleActionLimitHit(const std::string& msg)
 {
 	bool disable = true;
-	if ( _gui ) disable = _gui->yesno(msg);
-	else log_error("No gui registered, assuming 'Yes' answer to question: %s", msg);
+	if ( _interfaceHandler ) disable = _interfaceHandler->yesNo(msg);
+	else log_error("No user interface registered, assuming 'Yes' answer to "
+            "question: %s", msg);
 	if ( disable )
 	{
 		disableScripts();
@@ -295,7 +307,7 @@ movie_root::setLevel(unsigned int num, boost::intrusive_ptr<movie_instance> movi
 }
 
 void
-movie_root::swapLevels(boost::intrusive_ptr<sprite_instance> movie, int depth)
+movie_root::swapLevels(boost::intrusive_ptr<MovieClip> movie, int depth)
 {
 	assert(movie);
 
@@ -304,18 +316,23 @@ movie_root::swapLevels(boost::intrusive_ptr<sprite_instance> movie, int depth)
 	int oldDepth = movie->get_depth();
 
 #ifdef GNASH_DEBUG_LEVELS_SWAPPING
-	log_debug("Before swapLevels (source depth %d, target depth %d) levels are: ", oldDepth, depth);
+	log_debug("Before swapLevels (source depth %d, target depth %d) "
+            "levels are: ", oldDepth, depth);
 	for (Levels::const_iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
 	{
-		log_debug(" %d: %p (%s @ depth %d)", i->first, (void*)(i->second.get()), i->second->getTarget(), i->second->get_depth());
+		log_debug(" %d: %p (%s @ depth %d)", i->first,
+                (void*)(i->second.get()), i->second->getTarget(),
+                i->second->get_depth());
 	}
 #endif
 
 	if ( oldDepth < character::staticDepthOffset ) // should include _level0 !
 	{
 		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("%s.swapDepth(%d): movie has a depth (%d) below static depth zone (%d), won't swap its depth"),
-			movie->getTarget(), depth, oldDepth, character::staticDepthOffset);
+		log_aserror(_("%s.swapDepth(%d): movie has a depth (%d) below "
+                "static depth zone (%d), won't swap its depth"),
+                movie->getTarget(), depth, oldDepth,
+                character::staticDepthOffset);
 		);
 		return;
 	}
@@ -323,8 +340,10 @@ movie_root::swapLevels(boost::intrusive_ptr<sprite_instance> movie, int depth)
 	if ( oldDepth >= 0 ) 
 	{
 		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("%s.swapDepth(%d): movie has a depth (%d) below static depth zone (%d), won't swap its depth"),
-			movie->getTarget(), depth, oldDepth, character::staticDepthOffset);
+		log_aserror(_("%s.swapDepth(%d): movie has a depth (%d) below "
+                "static depth zone (%d), won't swap its depth"),
+                movie->getTarget(), depth, oldDepth,
+                character::staticDepthOffset);
 		);
 		return;
 	}
@@ -348,7 +367,7 @@ movie_root::swapLevels(boost::intrusive_ptr<sprite_instance> movie, int depth)
 	}
 	else
 	{
-		boost::intrusive_ptr<sprite_instance> otherMovie = targetIt->second;
+		boost::intrusive_ptr<MovieClip> otherMovie = targetIt->second;
 		otherMovie->set_depth(oldDepth);
 		oldIt->second = otherMovie;
 		targetIt->second = movie;
@@ -358,7 +377,9 @@ movie_root::swapLevels(boost::intrusive_ptr<sprite_instance> movie, int depth)
 	log_debug("After swapLevels levels are: ");
 	for (Levels::const_iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
 	{
-		log_debug(" %d: %p (%s @ depth %d)", i->first, (void*)(i->second.get()), i->second->getTarget(), i->second->get_depth());
+		log_debug(" %d: %p (%s @ depth %d)", i->first, 
+                (void*)(i->second.get()), i->second->getTarget(),
+                i->second->get_depth());
 	}
 #endif
 	
@@ -377,11 +398,12 @@ movie_root::dropLevel(int depth)
 	Levels::iterator it = _movies.find(depth);
 	if ( it == _movies.end() )
 	{
-		log_error("movie_root::dropLevel called against a movie not found in the levels container");
+		log_error("movie_root::dropLevel called against a movie not "
+                "found in the levels container");
 		return;
 	}
 
-	sprite_instance* mo = it->second.get();
+	MovieClip* mo = it->second.get();
 	if ( mo == getRootMovie() )
 	{
 		IF_VERBOSE_ASCODING_ERRORS(
@@ -401,8 +423,9 @@ movie_root::dropLevel(int depth)
 bool
 movie_root::loadLevel(unsigned int num, const URL& url)
 {
-	boost::intrusive_ptr<movie_definition> md ( create_library_movie(url) );
-	if (md == NULL)
+	boost::intrusive_ptr<movie_definition> md (
+            create_library_movie(url, _runInfo));
+	if (!md)
 	{
 		log_error(_("can't create movie_definition for %s"),
 			url.str());
@@ -419,7 +442,7 @@ movie_root::loadLevel(unsigned int num, const URL& url)
 	}
 
 	// Parse query string
-	sprite_instance::VariableMap vars;
+	MovieClip::VariableMap vars;
 	url.parse_querystring(url.querystring(), vars);
 	extern_movie->setVariables(vars);
 
@@ -444,7 +467,7 @@ movie_root::getLevel(unsigned int num) const
 void
 movie_root::reset()
 {
-	media::sound_handler* sh = get_sound_handler();
+	sound::sound_handler* sh = _runInfo.soundHandler();
 	if ( sh ) sh->reset();
 	clear();
 	_disableScripts = false;
@@ -467,6 +490,9 @@ movie_root::clear()
 	// wipe out all levels
 	_movies.clear();
 
+	// wipe out all childs
+	_childs.clear();
+
 	// remove all intervals
 	clearIntervalTimers();
 
@@ -485,7 +511,21 @@ movie_root::clear()
 	setInvalidated();
 }
 
-boost::intrusive_ptr<Stage>
+as_object*
+movie_root::getSelectionObject() const
+{
+    as_object* global = _vm.getGlobal();
+    if (!global) return 0;
+
+    as_value s;
+    if (!global->get_member(NSV::CLASS_SELECTION, &s)) return 0;
+    
+    as_object* sel = s.to_object().get();
+   
+    return sel;
+}
+
+boost::intrusive_ptr<Stage_as>
 movie_root::getStageObject()
 {
 	as_value v;
@@ -493,7 +533,7 @@ movie_root::getStageObject()
 	as_object* global = _vm.getGlobal();
 	if ( ! global ) return NULL;
 	if (!global->get_member(NSV::PROP_iSTAGE, &v) ) return NULL;
-	return boost::dynamic_pointer_cast<Stage>(v.to_object());
+	return boost::dynamic_pointer_cast<Stage_as>(v.to_object());
 }
 		
 void
@@ -509,7 +549,7 @@ movie_root::set_display_viewport(int x0, int y0, int w, int h)
 	if ( _scaleMode == noScale ) // rescale not allowed, notify Stage (if any)
 	{
 		//log_debug("Rescaling disabled");
-		boost::intrusive_ptr<Stage> stage = getStageObject();
+		boost::intrusive_ptr<Stage_as> stage = getStageObject();
 		if ( stage ) stage->notifyResize();
 	}
 
@@ -528,7 +568,7 @@ movie_root::notify_mouse_moved(int x, int y)
 
 }
 
-boost::intrusive_ptr<key_as_object>
+boost::intrusive_ptr<Key_as>
 movie_root::getKeyObject()
 {
 	// TODO: test what happens with the global "Key" object
@@ -542,15 +582,14 @@ movie_root::getKeyObject()
 		// TODO: use a named string...
 
 		as_value kval;
-		as_object* global = VM::get().getGlobal();
+		as_object* global = _vm.getGlobal();
 
-		std::string objName = PROPNAME("Key");
-		if (global->get_member(_vm.getStringTable().find(objName), &kval) )
+		if (global->get_member(NSV::CLASS_KEY, &kval) )
 		{
 			//log_debug("Found member 'Key' in _global: %s", kval.to_string());
 			boost::intrusive_ptr<as_object> obj = kval.to_object();
 			//log_debug("_global.Key to_object() : %s @ %p", typeid(*obj).name(), obj);
-			_keyobject = boost::dynamic_pointer_cast<key_as_object>( obj );
+			_keyobject = boost::dynamic_pointer_cast<Key_as>( obj );
 		}
 	}
 
@@ -567,8 +606,7 @@ movie_root::getMouseObject()
 		as_value val;
 		as_object* global = _vm.getGlobal();
 
-		std::string objName = PROPNAME("Mouse");
-		if (global->get_member(_vm.getStringTable().find(objName), &val) )
+		if (global->get_member(NSV::CLASS_MOUSE, &val) )
 		{
 			//log_debug("Found member 'Mouse' in _global: %s", val);
 			_mouseobject = val.to_object();
@@ -579,16 +617,16 @@ movie_root::getMouseObject()
 }
 
 
-key_as_object *
+Key_as *
 movie_root::notify_global_key(key::code k, bool down)
 {
-	if ( _vm.getSWFVersion() < 5 )
-	{
-		// Key was added in SWF5
-		return NULL; 
-	}
+    // NOTE: we don't check SWF version here
+    //       because even if the top-level movie was
+    //       an SWF4, it could have loaded an SWF5+
+    //       which would need to query Key object.
+    //       Testcase: http://www.ferryhalim.com/orisinal/g3/00dog.swf 
 
-	boost::intrusive_ptr<key_as_object> keyobject = getKeyObject();
+	boost::intrusive_ptr<Key_as> keyobject = getKeyObject();
 	if ( keyobject )
 	{
 		if (down) _keyobject->set_key_down(k);
@@ -596,7 +634,8 @@ movie_root::notify_global_key(key::code k, bool down)
 	}
 	else
 	{
-		log_error("gnash::notify_key_event(): _global.Key doesn't exist, or isn't the expected built-in");
+		log_error("gnash::notify_key_event(): _global.Key doesn't "
+				"exist, or isn't the expected built-in");
 	}
 
 	return _keyobject.get();
@@ -608,7 +647,7 @@ movie_root::notify_key_event(key::code k, bool down)
 	//
 	// First of all, notify the _global.Key object about key event
 	//
-	key_as_object * global_key = notify_global_key(k, down);
+	Key_as * global_key = notify_global_key(k, down);
 
 	// Notify character key listeners for clip key events
 	notify_key_listeners(k, down);
@@ -786,35 +825,16 @@ movie_root::generate_mouse_button_events()
 	        {
 		        // onPress
 
-		        // set/kill focus for current root
-		        character* current_active_entity = getFocus();
+                // Try setting focus on the new character. This will handle
+                // all necessary events and removal of current focus.
+                // Do not set focus to NULL.
+                if (ms.activeEntity) {
+                    setFocus(ms.activeEntity);
 
-		        // It's another entity ?
-		        if (current_active_entity != ms.activeEntity.get())
-		        {
-			        // First to clean focus
-			        if (current_active_entity != NULL)
-			        {
-				        current_active_entity->on_event(event_id::KILLFOCUS);
-				        need_redisplay=true;
-				        setFocus(NULL);
-			        }
-
-			        // Then to set focus
-			        if (ms.activeEntity)
-			        {
-				        if (ms.activeEntity->on_event(event_id::SETFOCUS))
-				        {
-					        setFocus(ms.activeEntity.get());
-				        }
-			        }
-		        }
-
-		        if (ms.activeEntity)
-		        {
 			        ms.activeEntity->on_button_event(event_id::PRESS);
 			        need_redisplay=true;
 		        }
+
 		        ms.wasInsideActiveEntity = true;
 		        ms.previousButtonState = MouseButtonState::DOWN;
 	        }
@@ -841,7 +861,7 @@ movie_root::fire_mouse_event()
     m_mouse_button_state.currentButtonState = (m_mouse_buttons & 1);
 
     // Set _droptarget if dragging a sprite
-    sprite_instance* dragging = 0;
+    MovieClip* dragging = 0;
     character* draggingChar = getDraggingCharacter();
     if ( draggingChar ) dragging = draggingChar->to_movie();
     if ( dragging )
@@ -880,7 +900,8 @@ movie_root::fire_mouse_event()
 }
 
 void
-movie_root::get_mouse_state(int& x, int& y, int& buttons)
+movie_root::get_mouse_state(boost::int32_t& x, boost::int32_t& y,
+        boost::int32_t& buttons)
 {
 //	    GNASH_REPORT_FUNCTION;
 
@@ -915,7 +936,7 @@ movie_root::set_drag_state(const drag_state& st)
 	{
 		// Get coordinates of the character's origin
 		point origin(0, 0);
-		matrix chmat = ch->get_world_matrix();
+		SWFMatrix chmat = ch->getWorldMatrix();
 		point world_origin;
 		chmat.transform(&world_origin, origin);
 
@@ -945,16 +966,16 @@ movie_root::doMouseDrag()
 		return; 
 	}
 
-	int	x, y, buttons;
+	boost::int32_t x, y, buttons;
 	get_mouse_state(x, y, buttons);
 
 	point world_mouse(PIXELS_TO_TWIPS(x), PIXELS_TO_TWIPS(y));
 
-	matrix	parent_world_mat;
+	SWFMatrix	parent_world_mat;
 	character* parent = dragChar->get_parent();
 	if (parent != NULL)
 	{
-	    parent_world_mat = parent->get_world_matrix();
+	    parent_world_mat = parent->getWorldMatrix();
 	}
 
 	if (! m_drag_state.isLockCentered())
@@ -976,9 +997,9 @@ movie_root::doMouseDrag()
 	// Place our origin so that it coincides with the mouse coords
 	// in our parent frame.
 	// TODO: add a character::set_translation ?
-	matrix	local = dragChar->get_matrix();
+	SWFMatrix	local = dragChar->getMatrix();
 	local.set_translation(world_mouse.x, world_mouse.y);
-	dragChar->set_matrix(local); //no need to update caches when only changing translation
+	dragChar->setMatrix(local); //no need to update caches when only changing translation
 }
 
 
@@ -1122,11 +1143,11 @@ movie_root::display()
 
 	for (Levels::iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
 	{
-		boost::intrusive_ptr<sprite_instance> movie = i->second;
+		boost::intrusive_ptr<MovieClip> movie = i->second;
 
 		movie->clear_invalidated();
 
-		if (movie->get_visible() == false) continue;
+		if (movie->isVisible() == false) continue;
 
 		// null frame size ? don't display !
 		const rect& sub_frame_size = movie->get_frame_size();
@@ -1141,6 +1162,18 @@ movie_root::display()
 
 	}
 
+	for (Childs::iterator i=_childs.begin(), e=_childs.end(); i!=e; ++i)
+	{
+		character* ch = i->second;
+
+		ch->clear_invalidated();
+
+		if (ch->isVisible() == false) continue;
+
+		ch->display();
+
+	}
+
 	render::end_display();
 }
 
@@ -1231,7 +1264,8 @@ void movie_root::notify_key_listeners(key::code k, bool down)
 }
 
 /* static private */
-void movie_root::add_listener(CharacterList& ll, character* listener)
+void
+movie_root::add_listener(CharacterList& ll, character* listener)
 {
 	assert(listener);
 	for(CharacterList::const_iterator i = ll.begin(), e = ll.end(); i != e; ++i)
@@ -1290,7 +1324,8 @@ movie_root::notify_mouse_listeners(const event_id& event)
             // Can throw an action limit exception if the stack limit is 0 or 1.
             // A stack limit like that is hardly of any use, but could be used
             // maliciously to crash Gnash.
-		    mouseObj->callMethod(NSV::PROP_BROADCAST_MESSAGE, as_value(PROPNAME(event.get_function_name())));
+		    mouseObj->callMethod(NSV::PROP_BROADCAST_MESSAGE, 
+                    event.functionName());
 		}
 	    catch (ActionLimitException &e)
 	    {
@@ -1309,18 +1344,61 @@ movie_root::notify_mouse_listeners(const event_id& event)
 	}
 }
 
-character*
+boost::intrusive_ptr<character>
 movie_root::getFocus()
 {
 	assert(testInvariant());
-	return m_active_input_text;
+	return _currentFocus;
 }
 
-void
-movie_root::setFocus(character* ch)
+bool
+movie_root::setFocus(boost::intrusive_ptr<character> to)
 {
-	m_active_input_text = ch;
+
+    // Nothing to do if current focus is the same as the new focus. 
+    // _level0 also seems unable to receive focus under any circumstances
+    // TODO: what about _level1 etc ?
+    if (to == _currentFocus || to == static_cast<character*>(getRootMovie())) {
+        return false;
+    }
+
+    if (to && !to->handleFocus()) {
+        // TODO: not clear whether to remove focus in this case.
+        return false;
+    }
+
+    // Undefined or NULL character removes current focus. Otherwise, try
+    // setting focus to the new character. If it fails, remove current
+    // focus anyway.
+
+    // Store previous focus, as the focus needs to change before onSetFocus
+    // is called and listeners are notified.
+    character* from = _currentFocus.get();
+
+    if (from) {
+
+        // Perform any actions required on killing focus (only TextField).
+        from->killFocus();
+        from->callMethod(NSV::PROP_ON_KILL_FOCUS, to.get());
+    }
+
+    _currentFocus = to;
+
+    if (to) {
+        to->callMethod(NSV::PROP_ON_SET_FOCUS, from);
+    }
+
+    as_object* sel = getSelectionObject();
+
+    /// Notify Selection listeners with previous and new focus as arguments.
+    if (sel) {
+        sel->callMethod(NSV::PROP_BROADCAST_MESSAGE, "onSetFocus",
+                from, to.get());
+    }
+
 	assert(testInvariant());
+
+    return true;
 }
 
 character*
@@ -1359,6 +1437,24 @@ movie_root::isMouseOverActiveEntity() const
     }
 }
 
+void
+movie_root::setQuality(Quality q)
+{
+    gnash::RcInitFile& rcfile = gnash::RcInitFile::getDefaultInstance();
+
+    /// Overridden quality if not negative.
+    if (rcfile.qualityLevel() >= 0) {
+        int ql = rcfile.qualityLevel();
+        ql = std::min<int>(ql, QUALITY_BEST);
+        _quality = static_cast<Quality>(ql);
+    }
+    else {
+        _quality = q;
+    }
+    render_handler* renderer = get_render_handler();
+    if (renderer) renderer->setQuality(_quality);
+}
+
 /// Get actionscript width of stage, in pixels. The width
 /// returned depends on the scale mode.
 unsigned int
@@ -1393,7 +1489,7 @@ void
 movie_root::setStageAlignment(short s)
 {
     _alignMode = s;
-    callInterface("Stage.align", "");
+    callInterface("Stage.align");
 }
 
 /// Returns a pair of enum values giving the actual alignment
@@ -1454,11 +1550,11 @@ movie_root::setStageScaleMode(ScaleMode sm)
     }
 
     _scaleMode = sm;
-    callInterface("Stage.align", "");    
+    callInterface("Stage.align");    
 
     if ( notifyResize )
     {
-        boost::intrusive_ptr<Stage> stage = getStageObject();
+        boost::intrusive_ptr<Stage_as> stage = getStageObject();
         if ( stage ) stage->notifyResize();
     }
 }
@@ -1468,7 +1564,7 @@ movie_root::setStageDisplayState(const DisplayState ds)
 {
     _displayState = ds;
 
-    boost::intrusive_ptr<Stage> stage = getStageObject();
+    boost::intrusive_ptr<Stage_as> stage = getStageObject();
     if ( stage ) stage->notifyFullScreen( (_displayState == fullScreen) );
 
 	if (!_interfaceHandler) return; // No registered callback
@@ -1496,18 +1592,14 @@ movie_root::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
 	{
 		i->second->add_invalidated_bounds(ranges, force);
 	}
-}
 
-void 
-movie_root::dump_character_tree() const 
-{
-  for (Levels::const_iterator i=_movies.begin(), e=_movies.end(); i!=e; ++i)
+	for (Childs::reverse_iterator i=_childs.rbegin(), e=_childs.rend(); i!=e; ++i)
 	{
-	  log_debug("--- movie at depth %d:", i->second->get_depth());
-		i->second->dump_character_tree("CTREE: ");
+		i->second->add_invalidated_bounds(ranges, force);
 	}
 }
 
+
 int
 movie_root::minPopulatedPriorityQueue() const
 {
@@ -1697,7 +1789,7 @@ movie_root::executeTimers()
         log_debug("Checking %d timers for expiration", _intervalTimers.size());
 #endif
 
-	unsigned long now = VM::get().getTime();
+	unsigned long now = _vm.getTime();
 
 	typedef std::multimap<unsigned int, Timer*> ExpiredTimers;
 	ExpiredTimers expiredTimers;
@@ -1758,7 +1850,15 @@ void
 movie_root::markReachableResources() const
 {
     // Mark movie levels as reachable
-    for (Levels::const_reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
+    for (Levels::const_reverse_iterator i=_movies.rbegin(), e=_movies.rend();
+            i!=e; ++i)
+    {
+        i->second->setReachable();
+    }
+
+    // Mark childs as reachable
+    for (Childs::const_reverse_iterator i=_childs.rbegin(), e=_childs.rend();
+            i!=e; ++i)
     {
         i->second->setReachable();
     }
@@ -1771,8 +1871,8 @@ movie_root::markReachableResources() const
     m_mouse_button_state.markReachableResources();
     
     // Mark timer targets
-    for (TimerMap::const_iterator i=_intervalTimers.begin(), e=_intervalTimers.end();
-            i != e; ++i)
+    for (TimerMap::const_iterator i=_intervalTimers.begin(),
+            e=_intervalTimers.end(); i != e; ++i)
     {
         i->second->markReachableResources();
     }
@@ -1794,6 +1894,8 @@ movie_root::markReachableResources() const
     // Mark global Mouse object
     if ( _mouseobject ) _mouseobject->setReachable();
 
+    if (_currentFocus) _currentFocus->setReachable();
+
     // Mark character being dragged, if any
     m_drag_state.markReachableResources();
 
@@ -1802,7 +1904,8 @@ movie_root::markReachableResources() const
     //       parent.
     //std::for_each(_liveChars.begin(), _liveChars.end(), boost::bind(&character::setReachable, _1));
 #if GNASH_PARANOIA_LEVEL > 1
-    for (LiveChars::const_iterator i=_liveChars.begin(), e=_liveChars.end(); i!=e; ++i)
+    for (LiveChars::const_iterator i=_liveChars.begin(), e=_liveChars.end();
+            i!=e; ++i)
     {
         assert((*i)->isReachable());
     }
@@ -1834,17 +1937,30 @@ movie_root::markReachableResources() const
 character *
 movie_root::getTopmostMouseEntity(boost::int32_t x, boost::int32_t y)
 {
+
+	for (Childs::reverse_iterator i=_childs.rbegin(), e=_childs.rend(); i!=e; ++i)
+	{
+		character* ret = i->second->get_topmost_mouse_entity(x, y);
+		if ( ret ) return ret;
+	}
+
 	for (Levels::reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
 	{
 		character* ret = i->second->get_topmost_mouse_entity(x, y);
 		if ( ret ) return ret;
 	}
+
 	return NULL;
 }
 
 const character *
 movie_root::findDropTarget(boost::int32_t x, boost::int32_t y, character* dragging) const
 {
+	for (Childs::const_reverse_iterator i=_childs.rbegin(), e=_childs.rend(); i!=e; ++i)
+	{
+		const character* ret = i->second->findDropTarget(x, y, dragging);
+		if ( ret ) return ret;
+	}
 	for (Levels::const_reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
 	{
 		const character* ret = i->second->findDropTarget(x, y, dragging);
@@ -1871,7 +1987,7 @@ movie_root::cleanupDisplayList()
         //       method of each sprite, but that will introduce 
         //       problems when we'll implement skipping ::display()
         //       when late on FPS. Alternatively we may have the
-        //       sprite_instance::markReachableResources take care
+        //       MovieClip::markReachableResources take care
         //       of cleaning up unloaded... but that will likely
         //       introduce problems when allowing the GC to run
         //       at arbitrary times.
@@ -1879,6 +1995,11 @@ movie_root::cleanupDisplayList()
         //       in local display lists must happen at the *end* of global action
         //       queue processing.
         //
+        for (Childs::reverse_iterator i=_childs.rbegin(), e=_childs.rend(); i!=e; ++i)
+        {
+                MovieClip* mc = dynamic_cast<MovieClip*>(i->second);
+                if ( mc ) mc->cleanupDisplayList();
+        }
         for (Levels::reverse_iterator i=_movies.rbegin(), e=_movies.rend(); i!=e; ++i)
         {
                 i->second->cleanupDisplayList();
@@ -1905,6 +2026,7 @@ movie_root::cleanupDisplayList()
 		int cleaned =0;
 #endif
 		needScan=false;
+
 		// Remove unloaded characters from the _liveChars list
 		for (LiveChars::iterator i=_liveChars.begin(), e=_liveChars.end(); i!=e;)
 		{
@@ -1941,6 +2063,7 @@ movie_root::cleanupDisplayList()
 				++i;
 			}
 		}
+
 #ifdef GNASH_DEBUG_DLIST_CLEANUP
 		cout << " Scan " << scansCount << " cleaned " << cleaned << " instances" << endl;
 #endif
@@ -2049,9 +2172,141 @@ movie_root::findCharacterByTarget(const std::string& tgtstr_orig) const
 }
 
 void
-movie_root::loadMovie(const URL& url, const std::string& target, const std::string* postdata)
+movie_root::getURL(const std::string& urlstr, const std::string& target,
+        const std::string& data, MovieClip::VariablesMethod method)
 {
+
+    if (_hostfd == -1)
+    {
+        /// If there is no hosting application, call the URL launcher. For
+        /// safety, we resolve the URL against the base URL for this run.
+        /// The data is not sent at all.
+        URL url(urlstr, _runInfo.baseURL());
+
+        gnash::RcInitFile& rcfile = gnash::RcInitFile::getDefaultInstance();
+        std::string command = rcfile.getURLOpenerFormat();
+
+        /// Try to avoid letting flash movies execute
+        /// arbitrary commands (sic)
+        ///
+        /// Maybe we should exec here, but if we do we might have problems
+        /// with complex urlOpenerFormats like:
+        ///    firefox -remote 'openurl(%u)'
+        ///
+        ///
+        /// NOTE: this escaping implementation is far from optimal, but
+        ///       I felt pretty in rush to fix the arbitrary command
+        ///      execution... we'll optimize if needed
+        ///
+        std::string safeurl = url.str(); 
+        boost::replace_all(safeurl, "\\", "\\\\");    // escape backslashes first
+        boost::replace_all(safeurl, "'", "\\'");    // then single quotes
+        boost::replace_all(safeurl, "\"", "\\\"");    // double quotes
+        boost::replace_all(safeurl, ";", "\\;");    // colons
+        boost::replace_all(safeurl, " ", "\\ ");    // spaces
+        boost::replace_all(safeurl, ">", "\\>");    // output redirection
+        boost::replace_all(safeurl, "<", "\\<");    // input redirection
+        boost::replace_all(safeurl, "&", "\\&");    // background (sic)
+        boost::replace_all(safeurl, "\n", "\\n");    // newline
+        boost::replace_all(safeurl, "\r", "\\r");    // return
+        boost::replace_all(safeurl, "\t", "\\t");    // tab
+        boost::replace_all(safeurl, "|", "\\|");    // pipe
+        boost::replace_all(safeurl, "`", "\\`");    // backtick
+
+        boost::replace_all(safeurl, "(", "\\(");    // subshell :'(
+        boost::replace_all(safeurl, ")", "\\)");    // 
+        boost::replace_all(safeurl, "}", "\\}");    // 
+        boost::replace_all(safeurl, "{", "\\{");    // 
+
+        boost::replace_all(safeurl, "$", "\\$");    // variable expansions
+
+        boost::replace_all(command, "%u", safeurl);
+
+        log_debug (_("Launching URL: %s"), command);
+        std::system(command.c_str());
+        return;
+    }
+
+    /// This is when there is a hosting application.
+    std::ostringstream request;
+    std::string querystring;
+    switch (method)
+    {
+        case MovieClip::METHOD_POST:
+             request << "POST " << target << ":" << 
+                data << "$" << urlstr << std::endl;
+             break;
+
+        // METHOD_GET and METHOD_NONE are the same, except that
+        // for METHOD_GET we append the variables to the query
+        // string.
+        case MovieClip::METHOD_GET:
+            // Append vars to URL query string
+            if (urlstr.find("?") == std::string::npos) {
+                querystring = "?";
+            }
+            else querystring = "&";
+            querystring.append(data);
+
+        case MovieClip::METHOD_NONE:
+            // use the original url, non parsed (the browser will know
+            // better how to resolve relative urls and handle
+            // javascript)
+            request << "GET " << target << ":" << urlstr << std::endl;
+            break;
+    }
+
+    std::string requestString = request.str();
+    size_t len = requestString.length();
+    // TODO: should mutex-protect this ?
+    // NOTE: we are assuming the hostfd is set in blocking mode here..
+    log_debug(_("Attempt to write geturl requests fd %d"), _hostfd);
+
+    int ret = write(_hostfd, requestString.c_str(), len);
+    if (ret == -1)
+    {
+        log_error(_("Could not write to user-provided host requests "
+                    "fd %d: %s"), _hostfd, std::strerror(errno));
+    }
+    if (static_cast<size_t>(ret) < len)
+    {
+        log_error(_("Could only write %d bytes over %d required to "
+                    "user-provided host requests fd %d"),
+                    ret, len, _hostfd);
+    }
+
+    // The request string ends with newline, and we don't want to log that
+    requestString.resize(requestString.size() - 1);
+    log_debug(_("Sent request '%s' to host fd %d"), requestString, _hostfd);
+
+}
+
+void
+movie_root::loadMovie(const std::string& urlstr, const std::string& target,
+        const std::string& data, MovieClip::VariablesMethod method)
+{
+
+    /// URL security is checked in StreamProvider::getStream() down the
+    /// chain.
+    URL url(urlstr, _runInfo.baseURL());
+
+    /// If the method is MovieClip::METHOD_NONE, we send no data.
+    if (method == MovieClip::METHOD_GET)
+    {
+        std::string varsToSend(urlstr);
+        /// GET: append data to query string.
+        std::string qs = url.querystring();
+        if ( qs.empty() ) varsToSend.insert(0, 1, '?');
+        else varsToSend.insert(0, 1, '&');
+        url.set_querystring(qs + varsToSend);
+    }
+
     log_debug("movie_root::loadMovie(%s, %s)", url.str(), target);
+
+    const std::string* postdata = NULL;
+
+    /// POST: send variables using the POST method.
+    if (method == MovieClip::METHOD_POST) postdata = &data;
     _loadMovieRequests.push_front(LoadMovieRequest(url, target, postdata));
 }
 
@@ -2063,23 +2318,26 @@ movie_root::processLoadMovieRequest(const LoadMovieRequest& r)
     bool usePost = r.usePost();
     const std::string& postData = r.getPostData();
 
-    if ( target.compare(0, 6, "_level") == 0 && target.find_first_not_of("0123456789", 7) == std::string::npos )
+    if (target.compare(0, 6, "_level") == 0 &&
+            target.find_first_not_of("0123456789", 7) == std::string::npos)
     {
-        unsigned int levelno = strtoul(target.c_str()+6, NULL, 0);
-        log_debug(_("processLoadMovieRequest: Testing _level loading (level %u)"), levelno);
+        unsigned int levelno = std::strtoul(target.c_str() + 6, NULL, 0);
+        log_debug(_("processLoadMovieRequest: Testing _level loading "
+                    "(level %u)"), levelno);
         loadLevel(levelno, url);
         return;
     }
 
     character* ch = findCharacterByTarget(target);
-    if ( ! ch )
+    if (!ch)
     {
-        log_debug("Target %s of a loadMovie request doesn't exist at processing time", target);
+        log_debug("Target %s of a loadMovie request doesn't exist at "
+                "processing time", target);
         return;
     }
 
-    sprite_instance* sp = ch->to_movie();
-    if ( ! sp )
+    MovieClip* sp = ch->to_movie();
+    if (!sp)
     {
         log_unimpl("loadMovie against a %s character", typeName(*ch));
         return;
@@ -2113,7 +2371,7 @@ movie_root::processLoadMovieRequests()
 bool
 movie_root::isLevelTarget(const std::string& name, unsigned int& levelno)
 {
-  if ( VM::get().getSWFVersion() > 6 )
+  if ( _vm.getSWFVersion() > 6 )
   {
     if ( name.compare(0, 6, "_level") ) return false;
   }
@@ -2150,9 +2408,6 @@ void
 movie_root::getMovieInfo(tree<StringPair>& tr, tree<StringPair>::iterator it)
 {
 
-    const std::string yes = _("yes");
-    const std::string no = _("no");
-
     tree<StringPair>::iterator localIter;
 
     //
@@ -2199,9 +2454,19 @@ movie_root::getMovieInfo(tree<StringPair>& tr, tree<StringPair>::iterator it)
     // Stage: scripts state (enabled/disabled)
     localIter = tr.append_child(it, StringPair("Scripts",
                 _disableScripts ? " disabled" : "enabled"));
-                
+     
+    getCharacterTree(tr, it);    
+}
+
+void
+movie_root::getCharacterTree(tree<StringPair>& tr,
+        tree<StringPair>::iterator it)
+{
+
+    tree<StringPair>::iterator localIter;
+
     /// Stage: number of live characters
-    os.str("");
+    std::ostringstream os;
     os << _liveChars.size();
     localIter = tr.append_child(it, StringPair(_("Live characters"), os.str()));
 
@@ -2213,6 +2478,7 @@ movie_root::getMovieInfo(tree<StringPair>& tr, tree<StringPair>::iterator it)
 	}
 
 }
+
 #endif
 
 void
@@ -2221,6 +2487,12 @@ movie_root::handleFsCommand(const std::string& cmd, const std::string& arg) cons
 	if ( _fsCommandHandler ) _fsCommandHandler->notify(cmd, arg);
 }
 
+void
+movie_root::errorInterface(const std::string& msg) const
+{
+	if (_interfaceHandler) _interfaceHandler->error(msg);
+}
+
 std::string
 movie_root::callInterface(const std::string& cmd, const std::string& arg) const
 {
@@ -2231,5 +2503,67 @@ movie_root::callInterface(const std::string& cmd, const std::string& arg) const
 	return "<no iface to hosting app>";
 }
 
+void
+movie_root::addChild(character* ch)
+{
+    int newDepth = _childs.empty() ? 0 : _childs.rbegin()->second->get_depth()+1;
+    ch->set_depth(newDepth);
+
+    assert(!_childs[newDepth]);
+
+    _childs[newDepth] = ch;
+
+    ch->set_invalidated();
+
+	/// Notify placement 
+	ch->stagePlacementCallback();
+}
+
+void
+movie_root::addChildAt(character* ch, int depth)
+{
+    setInvalidated();
+
+    // If this character already exist
+    // as a child, drop it first.
+	Childs::iterator existing = _childs.begin();
+    for (Childs::iterator end=_childs.end(); existing!=end; ++existing)
+    {
+        if ( existing->second == ch )
+        {
+            log_debug("Character %s found as child %d",
+                ch->getTarget(), existing->first);
+            _childs.erase(existing);
+            break;
+        }
+    }
+
+    ch->set_depth(depth);
+
+	Childs::iterator it = _childs.find(depth);
+	if ( it == _childs.end() ) {
+        _childs[depth] = ch;
+    } else {
+        if ( it->second == ch )
+        {
+            log_debug("Character %s already the child at depth %d",
+                ch->getTarget(), depth);
+        }
+        // don't leak overloaded childs
+        it->second->destroy();
+        it->second = ch;
+    }
+
+    if ( existing == _childs.end() )
+    {
+        ch->set_invalidated();
+
+        /// Notify placement 
+        ch->stagePlacementCallback();
+    }
+
+	assert(testInvariant());
+}
+
 } // namespace gnash
 
