@@ -1,6 +1,6 @@
 // ActionExec.cpp:  ActionScript execution, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -75,6 +75,7 @@ ActionExec::ActionExec(const swf_function& func, as_environment& newEnv, as_valu
     _initialStackSize(0),
     _initialCallStackDepth(0),
     _originalTarget(0),
+    _origExecSWFVersion(0),
     _tryList(),
     _returning(false),
     _abortOnUnload(false),
@@ -88,12 +89,25 @@ ActionExec::ActionExec(const swf_function& func, as_environment& newEnv, as_valu
     assert(stop_pc < code.size());
 
     // See comment in header
+    // TODO: stack limit dependent on function version or VM version ?
     if ( env.get_version() > 5 ) {
         _withStackLimit = 15;
     }
 
-    // SWF version 6 and higher pushes the activation object to the scope stack
-    if ( env.get_version() > 5 )
+    // Functions defined in SWF version 6 and higher pushes
+    // the activation object to the scope stack
+    // NOTE: if we query env.get_version() we get version of the calling
+    //       code, not the one we're about to call. This is because
+    //       it is ActionExec's call operator switching the VM version!
+    //
+    //       TESTCASE: winterbell from orisinal.
+    //       If we query VM version here, the movie results in:
+    //              set local var: i=[number:0]
+    //              push 'i' getvariable
+    //              get var: i=[undefined] 
+    // 
+    if ( code.getDefinitionVersion() > 5 )
+    //if ( env.get_version() > 5 )
     {
         // We assume that the swf_function () operator already initialized its environment
         // so that it's activation object is now in the top element of the CallFrame stack
@@ -113,6 +127,7 @@ ActionExec::ActionExec(const action_buffer& abuf, as_environment& newEnv, bool a
     _initialStackSize(0),
     _initialCallStackDepth(0),
     _originalTarget(0),
+    _origExecSWFVersion(0),
     _tryList(),
     _returning(false),
     _abortOnUnload(abortOnUnloaded),
@@ -126,6 +141,7 @@ ActionExec::ActionExec(const action_buffer& abuf, as_environment& newEnv, bool a
     //GNASH_REPORT_FUNCTION;
 
     /// See comment in header
+    // TODO: stack limit dependent on function version or VM version ?
     if ( env.get_version() > 5 ) {
         _withStackLimit = 15;
     }
@@ -134,9 +150,14 @@ ActionExec::ActionExec(const action_buffer& abuf, as_environment& newEnv, bool a
 void
 ActionExec::operator() ()
 {
+    VM& vm = env.getVM();
 
     // Do not execute if scripts are disabled
-    if ( VM::get().getRoot().scriptsDisabled() ) return;
+    if ( vm.getRoot().scriptsDisabled() ) return;
+
+    _origExecSWFVersion = vm.getSWFVersion();
+    int codeVersion = code.getDefinitionVersion();
+    vm.setSWFVersion(codeVersion);
 
     static const SWF::SWFHandlers& ash = SWF::SWFHandlers::instance();
         
@@ -149,8 +170,8 @@ ActionExec::operator() ()
 #if DEBUG_STACK
     IF_VERBOSE_ACTION (
             log_action(_("at ActionExec operator() start, pc=%d"
-                   ", stop_pc=%d, code.size=%d."),
-                pc, stop_pc, code.size());
+                   ", stop_pc=%d, code.size=%d, func=%d, codeVersion=%d"),
+                pc, stop_pc, code.size(), _func ? _func : 0, codeVersion);
         std::stringstream ss;
         env.dump_stack(ss, STACK_DUMP_LIMIT);
         env.dump_global_registers(ss);
@@ -264,7 +285,7 @@ ActionExec::operator() ()
                 break;
             }
 
-            ash.execute(static_cast<SWF::action_type>(action_id), *this);
+            ash.execute(static_cast<SWF::ActionType>(action_id), *this);
 
 #if 1 // See bugs: #20974, #21069, #20996.
 
@@ -279,7 +300,6 @@ ActionExec::operator() ()
 
             if ( _abortOnUnload && guardedChar && guardedChar->isUnloaded() )
             // action_execution_order_test8.c shows that the opcode guard is not SWF version based (TODO: automate it!)
-            // && VM::get().getSWFVersion() > 5 
             {
                 std::stringstream ss;
                 ss << "Target of action_buffer (" << guardedChar->getTarget() 
@@ -333,8 +353,9 @@ ActionExec::operator() ()
             {
                 if ( ++branchCount > maxBranchCount )
                 {
-                    boost::format fmt(_("Loop iterations count exceeded limit of "
-                                        "%d. Last branch was from pc %d to %d."));
+                    boost::format fmt(_("Loop iterations count exceeded "
+                                "limit of %d. Last branch was from pc %d "
+                                "to %d"));
                     fmt % maxBranchCount % oldPc % pc;
                     throw ActionLimitException(fmt.str());
                 }
@@ -577,10 +598,14 @@ ActionExec::processExceptions(TryBlock& t)
 void
 ActionExec::cleanupAfterRun(bool /*expectInconsistencies*/) // TODO: drop argument...
 {
+    VM& vm = env.getVM();
+
     //assert(_originalTarget); // this execution context might have been started while target had a null target
     env.set_target(_originalTarget);
     _originalTarget = NULL;
 
+    vm.setSWFVersion(_origExecSWFVersion);
+
     IF_VERBOSE_MALFORMED_SWF
     (
         // check if the stack was smashed
@@ -597,7 +622,7 @@ ActionExec::cleanupAfterRun(bool /*expectInconsistencies*/) // TODO: drop argume
     );
 
     // Have movie_root flush any newly pushed actions in higher priority queues
-    VM::get().getRoot().flushHigherPriorityActionQueues();
+    env.getVM().getRoot().flushHigherPriorityActionQueues();
 
 //    log_debug("After cleanup of ActionExec %p, env %p has "
 //        "stack size of %d and callStackDepth of %d",
@@ -671,13 +696,13 @@ ActionExec::pushWithEntry(const with_stack_entry& entry)
 bool
 ActionExec::delVariable(const std::string& name)
 {
-    return env.del_variable_raw(PROPNAME(name), getScopeStack());
+    return env.delVariableRaw(name, getScopeStack());
 }
 
 bool
 ActionExec::delObjectMember(as_object& obj, const std::string& name)
 {
-    string_table& st = VM::get().getStringTable();
+    string_table& st = env.getVM().getStringTable();
     std::pair<bool,bool> ret = obj.delProperty(st.find(PROPNAME(name)));
     return ret.second;
 }
@@ -716,14 +741,14 @@ ActionExec::setLocalVariable(const std::string& name, const as_value& val)
 void
 ActionExec::setObjectMember(as_object& obj, const std::string& var, const as_value& val)
 {
-    string_table& st = VM::get().getStringTable();
+    string_table& st = env.getVM().getStringTable();
     obj.set_member(st.find(PROPNAME(var)), val);
 }
 
 bool
 ActionExec::getObjectMember(as_object& obj, const std::string& var, as_value& val)
 {
-    string_table& st = VM::get().getStringTable();
+    string_table& st = env.getVM().getStringTable();
     return obj.get_member(st.find(PROPNAME(var)), &val);
 }
 
