@@ -1,6 +1,6 @@
 // MediaHandlerGst.h: GST media handler, for Gnash
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,18 +32,34 @@
 namespace gnash {
 namespace media {
 
+/// Gstreamer-based media handler module
+namespace gst {
+
+/// GST based MediaHandler
+//
+/// The module implements the MediaHandler factory as required
+/// by Gnash core for a loadable media handler module.
+///
+/// It uses gstreamer: http://gstreamer.freedesktop.org/
+///
+/// Starting point is MediaHandlerGst.
+/// 
 class DSOEXPORT MediaHandlerGst : public MediaHandler
 {
 public:
 
 	virtual std::auto_ptr<MediaParser> createMediaParser(std::auto_ptr<IOChannel> stream);
 
-	virtual std::auto_ptr<VideoDecoder> createVideoDecoder(VideoInfo& info);
+	virtual std::auto_ptr<VideoDecoder> createVideoDecoder(const VideoInfo& info);
 
-	virtual std::auto_ptr<AudioDecoder> createAudioDecoder(AudioInfo& info);
+	virtual std::auto_ptr<AudioDecoder> createAudioDecoder(const AudioInfo& info);
+	
+	virtual std::auto_ptr<VideoConverter> createVideoConverter(ImgBuf::Type4CC srcFormat,
+	                                                       ImgBuf::Type4CC dstFormat);
 };
 
 
+} // gnash.media.gst namespace
 } // gnash.media namespace 
 } // namespace gnash
 
