@@ -1,6 +1,6 @@
 // VideoDecoderGst.h: Video decoding using Gstreamer.
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,31 +17,36 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
-#ifndef __VIDEODECODERGST_H__
-#define __VIDEODECODERGST_H__
+#ifndef GNASH_VIDEODECODERGST_H
+#define GNASH_VIDEODECODERGST_H
 
 
 #ifdef HAVE_CONFIG_H
 #include "gnashconfig.h"
 #endif
 
-#include "image.h"
-#include <gst/gst.h>
+#include "GnashImage.h"
 #include "log.h"
-#include "MediaParser.h"
 #include "VideoDecoder.h"
 #include "dsodefs.h"
+#include "MediaParser.h" // for videoCodecType enum
+
+#include <gst/gst.h>
+
+
+#include "swfdec_codec_gst.h"
 
 
 namespace gnash {
 namespace media {
+namespace gst {
 
 // Convenience wrapper for GstBuffer. Intended to be wrapped in an auto_ptr.
-class gnashGstBuffer : public image::rgb
+class gnashGstBuffer : public ImageRGB
 {
 public:
   gnashGstBuffer(GstBuffer* buf, int width, int height)
-  : rgb(NULL, width, height, (width * 3 + 3) & ~3),
+  : ImageRGB(NULL, width, height, (width * 3 + 3) & ~3),
     _buffer(buf)
   {}
   
@@ -55,9 +60,13 @@ public:
     return GST_BUFFER_DATA(_buffer);
   }
 
-  std::auto_ptr<image::image_base> clone() const
+  const boost::uint8_t* data() const
   {
-    return std::auto_ptr<image_base>(new rgb(*this));
+    return GST_BUFFER_DATA(_buffer);
+  }
+  std::auto_ptr<GnashImage> clone() const
+  {
+    return std::auto_ptr<GnashImage>(new ImageRGB(*this));
   }
 
 private:
@@ -65,32 +74,46 @@ private:
 };
 
 
+/// GST based VideoDecoder
 class DSOEXPORT VideoDecoderGst : public VideoDecoder
 {
 public:
-  VideoDecoderGst(videoCodecType codec_type, int width, int height);
-  ~VideoDecoderGst();
+    VideoDecoderGst(videoCodecType codec_type, int width, int height,
+                    const boost::uint8_t* extradata, size_t extradatasize);
+    VideoDecoderGst(GstCaps* caps);
+    ~VideoDecoderGst();
 
-  void push(const EncodedVideoFrame& buffer);
+    void push(const EncodedVideoFrame& buffer);
 
-  std::auto_ptr<image::rgb> pop();
+    std::auto_ptr<GnashImage> pop();
   
-  bool peek();
+    bool peek();
 
-private:  
-  void checkMessages();
-  void handleMessage(GstMessage* message);
+    /// Get the width of the video
+    //
+    /// @return The width of the video in pixels or 0 if unknown.
+    int width() const;
 
-  VideoDecoderGst();
-  VideoDecoderGst(const gnash::media::VideoDecoderGst&);
+    /// Get the height of the video
+    //
+    /// @return The height of the video in pixels or 0 if unknown.
+    int height() const;
 
-  GstElement* _pipeline;
-  GstElement* _appsrc;
-  GstElement* _appsink;
-  GstElement* _colorspace;
+private:
+
+    int _width;
+    int _height;
+
+    void setup(GstCaps* caps);
+
+    VideoDecoderGst();
+    VideoDecoderGst(const VideoDecoderGst&);
+
+    SwfdecGstDecoder _decoder;
 };
 
 
+} // gnash.media.gst namespace
 } // namespace media
 } // namespace gnash
 #endif // __VIDEODECODERGST_H__
