@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,12 +22,14 @@
 
 #include <boost/thread/thread.hpp>
 #include <boost/thread/mutex.hpp>
+#include <boost/shared_ptr.hpp>
 #include <boost/bind.hpp>
 #include <algorithm>
 #include <string>
 #include <deque>
 #include <list>
 #include <map>
+#include <vector>
 
 #include "log.h"
 #include "network.h"
@@ -36,7 +38,6 @@
 #include "dsodefs.h" //For DSOEXPORT.
 
 #include "rtmp.h"
-#include "rtmp_server.h"
 #include "http.h"
 
 using namespace gnash;
@@ -57,51 +58,55 @@ Handler::Handler()
 Handler::~Handler()
 {
 //    GNASH_REPORT_FUNCTION;
-    closeConnection();
+//    closeConnection();
     _die = true;
-    notifyout();
+//    notifyout();
     notifyin();
 }
 
 bool
-Handler::push(amf::Buffer *data, fifo_e direction)
+Handler::push(boost::shared_ptr<amf::Buffer> data, fifo_e direction)
 {
 //    GNASH_REPORT_FUNCTION;
+#if 0
     if (direction == Handler::OUTGOING) {
-	_outgoing.push(data);
+      _outgoing.push(data);
 	return true;
     }
+#endif
     if (direction == Handler::INCOMING) {
-	_incoming.push(data);
+        _incoming.push(data);
 	return true;
     }
-    
+  
     return false;
 }
 
 // Push bytes on the outgoing FIFO
 bool
-Handler::push(gnash::Network::byte_t *data, int nbytes, fifo_e direction)
+Handler::push(boost::uint8_t *data, int nbytes, fifo_e direction)
 {
 //    GNASH_REPORT_FUNCTION;
-    amf::Buffer *ptr = new amf::Buffer;
+    boost::shared_ptr<amf::Buffer> ptr(new amf::Buffer);
     ptr->copy(data, nbytes);
     return push(ptr, direction);
 }
 
 // Pop the first date element off the FIFO
-amf::Buffer *
+boost::shared_ptr<amf::Buffer> 
 Handler::pop(fifo_e direction)
 {
 //    GNASH_REPORT_FUNCTION;
-    amf::Buffer *buf = NULL;
+    boost::shared_ptr<amf::Buffer> buf;
     
+#if 0
     if (direction == Handler::OUTGOING) {
 	if (_outgoing.size()) {
 	    buf = _outgoing.pop();
 	    return buf;	
 	}
     }
+#endif
     if (direction == Handler::INCOMING) {
 	if (_incoming.size()) {
 	    buf = _incoming.pop();
@@ -113,21 +118,23 @@ Handler::pop(fifo_e direction)
 }
 
 // Peek at the first data element without removing it
-amf::Buffer *
+boost::shared_ptr<amf::Buffer> 
 Handler::peek(fifo_e direction)
 {
 //    GNASH_REPORT_FUNCTION;
+#if 0
     if (direction == Handler::OUTGOING) {
 	if (_outgoing.size()) {
 	    return _outgoing.peek();
 	}
     }
+#endif
     if (direction == Handler::INCOMING) {
 	if (_incoming.size()) {
 	    return _incoming.peek();
 	}
     }    
-    return 0;
+    return boost::shared_ptr<amf::Buffer>();
 }
 
 // Return the size of the queues
@@ -135,9 +142,11 @@ size_t
 Handler::size(fifo_e direction)
 {
 //    GNASH_REPORT_FUNCTION;
+#if 0
     if (direction == Handler::OUTGOING) {
 	return _outgoing.size();
     }
+#endif
     if (direction == Handler::INCOMING) {
 	return _incoming.size();
     }
@@ -150,9 +159,11 @@ void
 Handler::clear(fifo_e direction)
 {
 //    GNASH_REPORT_FUNCTION;
+#if 0
     if (direction == Handler::OUTGOING) {
 	_outgoing.clear();
     }
+#endif
     if (direction == Handler::INCOMING) {
 	_incoming.clear();
     }    
@@ -164,56 +175,112 @@ Handler::dump()
 {
 //    GNASH_REPORT_FUNCTION;
     _incoming.dump();
-    _outgoing.dump();    
+//    _outgoing.dump();    
 }
 
+#if 0
+size_t
+Handler::readPacket(int fd)
+{
+    GNASH_REPORT_FUNCTION;
+    boost::shared_ptr<amf::Buffer> buf(new amf::Buffer);
+    if (fd <= 2) {
+	log_error("File discriptor out of range, %d", fd);
+	return -1;
+    }
+    
+    log_debug(_("Waiting for data on fd #%d..."), fd);
+    size_t ret = readNet(fd, buf->reference(), buf->size(), 1);
+    // the read timed out as there was no data, but the socket is still open.
+    if (ret == 0) {
+	log_debug("no data yet for fd #%d, continuing...", fd);
+	return 0;
+    }
+    // ret is "no position" when the socket is closed from the other end of the connection,
+    // so we're done.
+    if ((ret == string::npos) || (ret == 0xffffffff)) {
+	log_debug("socket for fd #%d was closed...", fd);
+	return -1;
+    }
+    // We got data. Resize the buffer if necessary.
+    if (ret > 0) {
+	if (ret < NETBUFSIZE) {
+	    buf->resize(ret);
+	    _incoming.push(buf);
+	}
+    } else {
+	log_debug("no more data for fd #%d, exiting...", fd);
+	die();
+	return -1;
+    }
+    return ret;
+}
+#endif
+
 // start the two thread handlers for the queues
 bool
-Handler::start(thread_params_t *args)
+Handler::start(Network::thread_params_t *args)
 {
     GNASH_REPORT_FUNCTION;
 //    Handler *hand = reinterpret_cast<Handler *>(args->handle);
     
     _incoming.setName("Incoming");
-    _outgoing.setName("Outgoing");
+//    _outgoing.setName("Outgoing");
     
     log_debug(_("Starting Handlers for port %d, tid %ld"),
 	      args->port, get_thread_id());
 
-    if (args->port == 4080) {			// FIXME: hack alert!
-	boost::thread handler(boost::bind(&httphandler, args));
+    struct pollfd fds; // FIXME: never initialized ?
+    int nfds = 1;
+    Network net;
+    boost::shared_ptr<vector<struct pollfd> > hits = net.waitForNetData(nfds, &fds);
+    vector<int>::const_iterator it;
+#if 0
+    for (it = _pollfds.begin(); it != _pollfds.end(); it++) {
+//	Buffer buf;
+//	net.readNet(*it, buf.reference(), buf.size());
+	args->netfd = *it;
+	if (crcfile.getThreadingFlag()) {
+	    if (args->port == port_offset + gnash::RTMPT_PORT) {
+		boost::thread handler(boost::bind(&http_handler, args));
+	    }
+	} else {
+	    callback[*it](args);
+	}
+    }    
+#endif
+    
+//     boost::thread outport(boost::bind(&netout_handler, args));
+//    boost::thread inport(boost::bind(&netin_handler, args));
+
+#if 0
+    if (args->port == 4080) {	// FIXME: hack alert!
+	boost::thread handler(boost::bind(&http_handler, args));
     }
     if (args->port == RTMP_PORT) {
 	boost::thread handler(boost::bind(&rtmp_handler, args));
     }
-    
-    boost::thread outport(boost::bind(&netout_handler, args));
-    boost::thread inport(boost::bind(&netin_handler, args));
-// We don't want to wait for the threads to complete, we
-// want to return to the main program so it can spawn another
-// thread for the next incoming connection.    
-// 	inport.join();    
-// 	outport.join();
-// 	handler.join();
-//     if (_die) {
-// 	log_debug("Handler done...");
-//     }
+#endif
+
     return true;
 }
-    
+
 extern "C" {
 void
-netin_handler(Handler::thread_params_t *args)
+netin_handler(Network::thread_params_t *args)
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
 
-    Handler *hand = reinterpret_cast<Handler *>(args->handle);
+    //Network *net = reinterpret_cast<Network *>(args->handler);
+    size_t ret=0;
 
     log_debug("Starting to wait for data in net for fd #%d", args->netfd);
     
     do {
-	amf::Buffer *buf = new amf::Buffer;
-	size_t ret = hand->readNet(args->netfd, buf->reference(), buf->size(), 1);
+	boost::shared_ptr<amf::Buffer> buf(new amf::Buffer);
+//	ret = hand->readNet(args->netfd, buf->reference(), buf->size(), 1);
+
+//	cerr << (char *)buf->reference() << endl;
 	// the read timed out as there was no data, but the socket is still open.
  	if (ret == 0) {
 	    log_debug("no data yet for fd #%d, continuing...", args->netfd);
@@ -227,26 +294,29 @@ netin_handler(Handler::thread_params_t *args)
 	}
 	// We got data. Resize the buffer if necessary.
 	if (ret > 0) {
-	    if (ret < NETBUFSIZE) {
-		buf->resize(ret);
-	    }
-	    hand->push(buf);
-	    hand->notify();
+//	    cerr << "XXXXX" << (char *)buf->reference() << endl;
+// 	    if (ret < NETBUFSIZE) {
+// 		buf->resize(ret);
+// 	    }
+//	    hand->push(buf);
+//	    hand->notify();
 	} else {
 	    log_debug("no more data for fd #%d, exiting...", args->netfd);
-	    hand->die();
+//	    hand->die();
 	    break;
 	}
-    } while (!hand->timetodie());
+//    } while (!hand->timetodie());
+    } while (ret > 0);
     // We're done. Notify the other threads the socket is closed, and tell them to die.
     log_debug("Net In handler done for fd #%d...", args->netfd);
-    hand->notify();
-    hand->closeNet(args->netfd);
+//    hand->notify();
+//    hand->closeNet(args->netfd);
 //    hand->dump();
 }
 
+#if 0
 void
-netout_handler(Handler::thread_params_t *args)
+netout_handler(Network::thread_params_t *args)
 {
 //    GNASH_REPORT_FUNCTION;
     int ret = 0;
@@ -261,7 +331,7 @@ netout_handler(Handler::thread_params_t *args)
 	}
 	hand->waitout();
 	while (hand->outsize()) {
-	    amf::Buffer *buf = hand->popout();
+	    boost::shared_ptr<amf::Buffer> buf = hand->popout();
 //	    log_debug("FIXME: got data in Outgoing que");
 //	    buf->dump();
 //	    ret = hand->writeNet(buf->reference(), buf->size(), 15);
@@ -269,7 +339,6 @@ netout_handler(Handler::thread_params_t *args)
 // 			log_debug("Got smaller packet, size %d", buf->size());		
 // 	    }
 	    ret = hand->writeNet(args->netfd, buf);
-	    delete buf;
 	}
     } while (ret > 0);
     hand->die();
@@ -286,7 +355,8 @@ netout_handler(Handler::thread_params_t *args)
     delete hand;
 #endif
 }
-    
+#endif
+
 } // end of extern C
 
 } // end of gnash namespace
