@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,11 +15,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 // 
-//
-//
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "log.h"
@@ -31,7 +29,7 @@
 
 #include <sys/stat.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#include "GnashSystemIOHeaders.h"
 #endif
 
 #include <regex.h>
@@ -61,19 +59,11 @@ bool nodelay = false;           // FIXME: this flag gets set based on
 				// can optimize it's own performance
 				// when needed,
 
-#ifdef HAVE_LIBXML
-extern int xml_fd;		// FIXME: this is the file descriptor
-				// from XMLSocket::connect(). This
-				// needs to be propogated up through
-				// the layers properly, but first I
-				// want to make sure it all works.
-#endif // HAVE_LIBXML
-
 TestState runtest;
 
 int
 main (int /*argc*/, char** /*argv*/) {
-    RcInitFile rc;
+    RcInitFile& rc = RcInitFile::getDefaultInstance();
 
     gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
     dbglogfile.setVerbosity();
@@ -158,12 +148,42 @@ main (int /*argc*/, char** /*argv*/) {
         runtest.fail ("enableExtensions");
     }
 
+    if (rc.getFlashVersionString() == "GSH 9,0,99,0") {
+        runtest.pass ("getFlashVersionString");
+    } else {
+        runtest.fail ("getFlashVersionString");
+    }
+
+    if (rc.getDebugLog() == "/tmp/gnash-dbg.log") {
+        runtest.pass ("getDebugLog");
+    } else {
+        runtest.fail ("getDebugLog");
+    }
+
     if (rc.startStopped() == 1) {
         runtest.pass ("startStopped");
     } else {
         runtest.fail ("startStopped");
     }
 
+    if (rc.getStreamsTimeout() == 1.5) {
+        runtest.pass ("streamsTimeout");
+    } else {
+        runtest.fail ("streamsTimeout");
+    }
+
+    if (rc.insecureSSL()) {
+        runtest.pass ("insecureSSL");
+    } else {
+        runtest.fail ("insecureSSL");
+    }
+
+    if (rc.getSOLSafeDir().size() > 0) {
+        runtest.pass ("getSOLSafeDir");
+    } else {
+        runtest.fail ("getSOLSafeDir");
+    }
+
     std::vector<std::string> whitelist = rc.getWhiteList();
     if (whitelist.size()) {
         if ((whitelist[0] == "www.doonesbury.com")
@@ -175,7 +195,7 @@ main (int /*argc*/, char** /*argv*/) {
         }
         runtest.pass ("rc.getWhiteList() has elements");
     } else {
-        runtest.fail ("rc.getWhiteList() doesn't has elements");        
+        runtest.fail ("rc.getWhiteList() doesn't have elements");        
     }
         
     std::vector<std::string> blacklist = rc.getBlackList();
@@ -188,7 +208,92 @@ main (int /*argc*/, char** /*argv*/) {
         }
         runtest.pass ("rc.getBlackList() has elements");
     } else {
-        runtest.fail ("rc.getBlackList() doesn't has elements");        
+        runtest.fail ("rc.getBlackList() doesn't have elements");        
+    }
+
+    const std::vector<std::string>& localSandbox = rc.getLocalSandboxPath();
+    if (localSandbox.size() != 1) {
+        runtest.fail ("rc.getLocalSandboxPath() doesn't have 1 element after set");
+    } else {
+        if ( localSandbox[0] == "/tmp/p1" )
+        {
+            runtest.pass ("set localSandbox");
+        }
+        else
+        {
+            runtest.fail ("rc.getLocalSandboxPath() doesn't have the correct first element after set");
+        }
+    }
+
+    if (rc.useXv() == false) {
+        runtest.pass ("Xvideo off by default");
+    } else {
+        runtest.fail ("Xvideo is on by default!");
+    }
+
+    // Parse a second file
+    if (rc.parseFile("gnashrc-local")) {
+
+	// Test whether blacklist in gnashrc-local is appended
+        std::vector<std::string> blacklist = rc.getBlackList();
+        if (blacklist.size()) {
+            if ((blacklist[2] == "www.gnashdev.org")
+                && (blacklist[3] == "www.wikipedia.de")) {
+                runtest.pass ("rc.getBlackList(): append");
+            } else {
+                runtest.fail ("rc.getBlackList(): append");
+            }
+            runtest.pass ("rc.getBlackList(): has appended elements");
+        } else {
+            runtest.fail ("rc.getBlackList(): doesn't appended elements");        
+        }
+
+        // Test local override of previous whitelist 
+        std::vector<std::string> whitelist = rc.getWhiteList();
+        if (whitelist.size()) {
+            runtest.fail ("rc.getWhiteList(): local override failed");
+        } else {
+            runtest.pass ("rc.getWhiteList(): local override succeeded");
+        }
+
+        // Test local override of previous local sandbox
+        const std::vector<std::string>& localSandbox = rc.getLocalSandboxPath();
+        if (localSandbox.empty()) {
+            runtest.fail ("rc.getLocalSandboxPath() doesn't have elements after append");        
+        } else {
+            if ( localSandbox.back() == "/tmp/gnash" )
+            {
+                runtest.pass ("append localSandbox");
+            }
+            else
+            {
+                runtest.fail ("rc.getLocalSandboxPath() doesn't have the correct last element after append");
+            }
+        }
+
+        if (rc.useXv() == true) {
+            runtest.pass ("Xvideo on by local.rc");
+        } else {
+            runtest.fail ("Xvideo on by local.rc");
+        }
+    }
+
+    if (rc.getURLOpenerFormat() == "lynx %u") {
+        runtest.pass ("getURLOpenerFormat");
+    } else {
+        runtest.fail ("getURLOpenerFormat");
+    }
+
+    if (rc.getSOLSafeDir() == "/tmp/SharedObjects") {
+        runtest.pass ("getSOLSafeDir");
+    } else {
+        runtest.fail ("getSOLSafeDir");
+    }
+
+    if (rc.getSOLReadOnly() == true) {
+        runtest.pass ("getSOLReadOnly");
+    } else {
+        runtest.fail ("getSOLReadOnly");
     }
 }
 
