@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,9 @@
 #define INPUT_FILENAME "RollOverOutTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "character.h"
-#include "dlist.h"
-#include "container.h"
+#include "DisplayList.h"
 #include "log.h"
 
 #include "check.h"
@@ -43,28 +42,23 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
-	check_equals(root->get_frame_count(), 3);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->get_frame_count(), 4);
+	check_equals(root->get_play_state(), MovieClip::PLAY);
 	check_equals(root->get_current_frame(), 0);
 
+	tester.advance();  // advance to the second frame.
+
 	const character* mc1 = tester.findDisplayItemByName(*root, "square1");
 	check(mc1);
 	const character* mc2 = tester.findDisplayItemByName(*root, "square2");
 	check(mc2);
-	check_equals(mc1->get_visible(), false);
-	check_equals(mc2->get_visible(), false);
 
-	tester.advance();
-
-	check_equals(mc1->get_visible(), true);
-	check_equals(mc2->get_visible(), false);
-
-	//check_equals(mc1->get_height(), 40*20);
-	//check_equals(mc1->get_width(), 40*20);
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(mc1->isVisible(), true);
+	check_equals(mc2->isVisible(), false);
+	check_equals(root->get_play_state(), MovieClip::STOP);
 	check_equals(root->get_current_frame(), 1);
 
 	// we're in stop mode, so advance should not advance anything
@@ -72,16 +66,14 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(root->get_current_frame(), 1);
 	tester.advance();
 	check_equals(root->get_current_frame(), 1);
-	tester.advance();
-	check_equals(root->get_current_frame(), 1);
 
 	// roll over the middle of the square, this should trigger
-	// the addition of a goto_frame(2) action, which is executed
+	// the addition of a goto_frame(3) action, which is executed
 	// at advance() time.
 	tester.movePointerTo(60, 60);
 	tester.advance();
 	check_equals(root->get_current_frame(), 2);
-	check_equals(root->get_play_state(), sprite_instance::STOP);
+	check_equals(root->get_play_state(), MovieClip::STOP);
 	tester.advance();
 	check_equals(root->get_current_frame(), 2);
 
@@ -94,6 +86,18 @@ main(int /*argc*/, char** /*argv*/)
 	tester.movePointerTo(40, 60);
 	tester.advance();
 	check_equals(root->get_current_frame(), 2);
-
+	
+	// pointer out of the square.
+	tester.movePointerTo(300, 60);
+	//tester.advance();  // mouse event handler should drive the movie
+	check_equals(root->get_current_frame(), 1);
+	
+	// pointer back to the square again.
+	tester.movePointerTo(60, 60);
+	tester.click();
+	//tester.advance(); // mouse event handler should drive the movie
+	check_equals(root->get_current_frame(), 3);
 }
 
+
+
