@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #define INPUT_FILENAME "attachMovieLoopingTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "character.h"
 #include "DisplayList.h"
 #include "log.h"
@@ -42,11 +42,11 @@ main(int /*argc*/, char** /*argv*/)
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
-	sprite_instance* root = tester.getRootMovie();
+	MovieClip* root = tester.getRootMovie();
 	assert(root);
 
 	check_equals(root->get_frame_count(), 2);
-	check_equals(root->get_play_state(), sprite_instance::PLAY);
+	check_equals(root->get_play_state(), MovieClip::PLAY);
 	check_equals(root->get_current_frame(), 0);
 
 	check(! tester.findDisplayItemByDepth(*root, 70) );
@@ -77,7 +77,7 @@ main(int /*argc*/, char** /*argv*/)
 	check(! tester.findDisplayItemByDepth(*root, 72) );
 	check(! tester.findDisplayItemByDepth(*root, 73) );
 
-	tester.movePointerTo(100, 30);
+	tester.movePointerTo(100, 5);
 	check(tester.isMouseOverMouseEntity());
 	tester.movePointerTo(170, 30);
 	check(!tester.isMouseOverMouseEntity());
@@ -90,7 +90,7 @@ main(int /*argc*/, char** /*argv*/)
 	check( tester.findDisplayItemByDepth(*root, 72) );
 	check(! tester.findDisplayItemByDepth(*root, 73) );
 
-	tester.movePointerTo(170, 30);
+	tester.movePointerTo(170, 10);
 	check(tester.isMouseOverMouseEntity());
 	tester.movePointerTo(240, 30);
 	check(!tester.isMouseOverMouseEntity());
@@ -103,7 +103,7 @@ main(int /*argc*/, char** /*argv*/)
 	check( tester.findDisplayItemByDepth(*root, 72) );
 	check( tester.findDisplayItemByDepth(*root, 73) );
 
-	tester.movePointerTo(240, 30);
+	tester.movePointerTo(240, 5);
 	check(tester.isMouseOverMouseEntity());
 }
 
