@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #define INPUT_FILENAME "loadMovieTest.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
+#include "MovieClip.h"
 #include "character.h"
 #include "DisplayList.h"
 #include "log.h"
@@ -32,19 +32,19 @@
 #include "check.h"
 #include <string>
 #include <cassert>
-#include <unistd.h>
+#include "GnashSystemIOHeaders.h"
 
 using namespace gnash;
 using namespace std;
 
 std::auto_ptr<MovieTester> tester;
-boost::intrusive_ptr<sprite_instance> root;
+boost::intrusive_ptr<MovieClip> root;
 
-sprite_instance*
+MovieClip*
 getCoverArt()
 {
 	character* coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
-	sprite_instance* coverart = coverartch->to_movie();
+	MovieClip* coverart = coverartch->to_movie();
 
 	//log_debug("Coverart is %p, displaylist is:", coverart);
 	//coverart->getDisplayList().dump();
@@ -55,7 +55,7 @@ getCoverArt()
 void
 checkScribbling()
 {
-	sprite_instance* coverart = getCoverArt();
+	MovieClip* coverart = getCoverArt();
 
 	size_t initial_child_count = coverart->getDisplayList().size();
 
@@ -104,7 +104,7 @@ main(int /*argc*/, char** /*argv*/)
 
 	// Verify that 'coverart' exists and is empty
 	character* coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
-	sprite_instance* coverart = coverartch->to_movie();
+	MovieClip* coverart = coverartch->to_movie();
 	check(coverart);
 	url = coverart->get_movie_definition()->get_url();
 	check_equals(coverart->get_movie_definition()->get_url(), baseURL.str());
